@@ -1,5 +1,5 @@
 #ifndef lint
-static char *RCSid() { return RCSid("$Id: axis.c,v 1.43 2004/04/13 17:23:51 broeker Exp $"); }
+static char *RCSid() { return RCSid("$Id: axis.c,v 1.60.2.1 2006/10/22 12:09:43 broeker Exp $"); }
 #endif
 
 /* GNUPLOT - axis.c */
@@ -38,6 +38,7 @@ static char *RCSid() { return RCSid("$Id: axis.c,v 1.43 2004/04/13 17:23:51 broe
 
 #include "stdfn.h"
 
+#include "alloc.h"
 #include "command.h"
 #include "gadgets.h"
 #include "gp_time.h"
@@ -60,19 +61,17 @@ AXIS axis_array[AXIS_ARRAY_SIZE]
 /* Keep defaults varying by axis in their own array, to ease initialization
  * of the main array */
 const AXIS_DEFAULTS axis_defaults[AXIS_ARRAY_SIZE] = {
-    { -10, 10, "z" , TICS_ON_BORDER,               }, 
-    { -10, 10, "y" , TICS_ON_BORDER | TICS_MIRROR, }, 
-    { -10, 10, "x" , TICS_ON_BORDER | TICS_MIRROR, }, 
-    { - 5,  5, "t" , NO_TICS,                      }, 
+    { -10, 10, "z" , TICS_ON_BORDER,               },
+    { -10, 10, "y" , TICS_ON_BORDER | TICS_MIRROR, },
+    { -10, 10, "x" , TICS_ON_BORDER | TICS_MIRROR, },
+    { - 5,  5, "t" , NO_TICS,                      },
     { -10, 10, "z2", NO_TICS,                      },
     { -10, 10, "y2", NO_TICS,                      },
     { -10, 10, "x2", NO_TICS,                      },
-    { - 0, 10, "r" , NO_TICS,                      }, 
-    { - 5,  5, "u" , NO_TICS,                      }, 
-    { - 5,  5, "v" , NO_TICS,                      }, 
-#ifdef PM3D
-    { -10, 10, "cb", TICS_ON_BORDER | TICS_MIRROR, }, 
-#endif
+    { - 0, 10, "r" , NO_TICS,                      },
+    { - 5,  5, "u" , NO_TICS,                      },
+    { - 5,  5, "v" , NO_TICS,                      },
+    { -10, 10, "cb", TICS_ON_BORDER | TICS_MIRROR, },
 };
 
 
@@ -109,12 +108,10 @@ const struct gen_table axisname_tbl[AXIS_ARRAY_SIZE + 1] =
     { "r", R_AXIS},
     { "u", U_AXIS},
     { "v", V_AXIS},
-#ifdef PM3D
     { "cb", COLOR_AXIS},
-#endif
     { NULL, -1}
 };
-    
+
 
 /* penalty for doing tics by callback in gen_tics is need for global
  * variables to communicate with the tic routines. Dont need to be
@@ -127,23 +124,15 @@ const struct gen_table axisname_tbl[AXIS_ARRAY_SIZE + 1] =
 
 const struct ticdef default_axis_ticdef = DEFAULT_AXIS_TICDEF;
 
-double ticscale = 1.0;		/* scale factor for tic mark */
-double miniticscale = 0.5;	/* and for minitics */
-TBOOLEAN tic_in = TRUE;		/* tics on inside of coordinate box? */
-
 /* axis labels */
-const label_struct default_axis_label = EMPTY_LABELSTRUCT;
+const text_label default_axis_label = EMPTY_LABELSTRUCT;
 
 /* zeroaxis drawing */
 const lp_style_type default_axis_zeroaxis = DEFAULT_AXIS_ZEROAXIS;
 
-/* grid drawing */  
+/* grid drawing */
 /* int grid_selection = GRID_OFF; */
-#ifdef PM3D
 # define DEFAULT_GRID_LP { 0, -1, 0, 1.0, 1.0, 0 }
-#else
-# define DEFAULT_GRID_LP { 0, -1, 0, 1.0, 1.0 }
-#endif
 const struct lp_style_type default_grid_lp = DEFAULT_GRID_LP;
 struct lp_style_type grid_lp   = DEFAULT_GRID_LP;
 struct lp_style_type mgrid_lp  = DEFAULT_GRID_LP;
@@ -171,6 +160,7 @@ static TBOOLEAN axis_position_zeroaxis __PROTO((AXIS_INDEX));
 static double quantize_duodecimal_tics __PROTO((double, int));
 static void get_position_type __PROTO((enum position_type * type, int *axes));
 
+
 /* ---------------------- routines ----------------------- */
 
 /* check range and take logs of min and max if logscale
@@ -178,14 +168,13 @@ static void get_position_type __PROTO((enum position_type * type, int *axes));
  */
 #define LOG_MSG(x) x " range must be greater than 0 for scale"
 
+/* {{{ axis_unlog_interval() */
+
 /* this is used in a few places all over the code: undo logscaling of
  * a given range if necessary. If checkrange is TRUE, will int_error() if
  * range is invalid */
 void
-axis_unlog_interval(axis, min, max, checkrange)
-    AXIS_INDEX axis;
-    double *min, *max;
-    TBOOLEAN checkrange;
+axis_unlog_interval(AXIS_INDEX axis, double *min, double *max, TBOOLEAN checkrange)
 {
     if (axis_array[axis].log) {
 	if (checkrange && (*min<= 0.0 || *max <= 0.0))
@@ -195,11 +184,14 @@ axis_unlog_interval(axis, min, max, checkrange)
 	*min = (*min<=0) ? -VERYLARGE : AXIS_DO_LOG(axis,*min);
 	*max = (*max<=0) ? -VERYLARGE : AXIS_DO_LOG(axis,*max);
     }
-} 
+}
+
+/* }}} */
+
+/* {{{ axis_revert_and_unlog_range() */
 
 void
-axis_revert_and_unlog_range(axis)
-    AXIS_INDEX axis;
+axis_revert_and_unlog_range(AXIS_INDEX axis)
 {
   if (axis_array[axis].range_is_reverted) {
     double temp = axis_array[axis].min;
@@ -209,13 +201,11 @@ axis_revert_and_unlog_range(axis)
   axis_unlog_interval(axis, &axis_array[axis].min, &axis_array[axis].max, 1);
 }
 
+/* }}} */
 
-/*{{{  axis_log_value_checked() */
+/* {{{ axis_log_value_checked() */
 double
-axis_log_value_checked(axis, coord, what)
-     AXIS_INDEX axis;
-     double coord;		/* the value */
-     const char *what;		/* what is the coord for? */
+axis_log_value_checked(AXIS_INDEX axis, double coord, const char *what)
 {
     if (axis_array[axis].log) {
 	if (coord <= 0.0) {
@@ -227,9 +217,9 @@ axis_log_value_checked(axis, coord, what)
     return (coord);
 }
 
-/*}}} */
+/* }}} */
 
-/*{{{  axis_checked_extend_empty_range() */
+/* {{{ axis_checked_extend_empty_range() */
 /*
  * === SYNOPSIS ===
  *
@@ -272,7 +262,7 @@ axis_log_value_checked(axis, coord, what)
  *
  * The current code plots all the above examples correctly and without
  * infinite looping.
- * 
+ *
  * HBB 2000/05/01: added an additional up-front test, active only if
  *   the new 'mesg' parameter is non-NULL.
  *
@@ -289,7 +279,7 @@ axis_log_value_checked(axis, coord, what)
  *             if it isn't.
  *
  * Global Variables:
- * auto_array, min_array, max_array (in out) (defined in axis.[ch]): 
+ * auto_array, min_array, max_array (in out) (defined in axis.[ch]):
  *    variables describing the status of autoscaling and range ends, for
  *    each of the possible axes.
  *
@@ -297,9 +287,7 @@ axis_log_value_checked(axis, coord, what)
  *
  */
 void
-axis_checked_extend_empty_range(axis,mesg)
-     AXIS_INDEX axis;
-     const char *mesg;
+axis_checked_extend_empty_range(AXIS_INDEX axis, const char *mesg)
 {
     /* These two macro definitions set the range-widening policy: */
 
@@ -325,9 +313,10 @@ axis_checked_extend_empty_range(axis,mesg)
 	if (axis_array[axis].autoscale) {
 	    /* range came from autoscaling ==> widen it */
 	    double widen = (dmax == 0.0) ?
-		FIXUP_RANGE__WIDEN_ZERO_ABS 
+		FIXUP_RANGE__WIDEN_ZERO_ABS
 		: FIXUP_RANGE__WIDEN_NONZERO_REL * dmax;
-	    fprintf(stderr, "Warning: empty %s range [%g:%g], ",
+	    if (!(axis == FIRST_Z_AXIS && !mesg)) /* set view map */
+		fprintf(stderr, "Warning: empty %s range [%g:%g], ",
 		    axis_defaults[axis].name, dmin, dmax);
 	    /* HBB 20010525: correctly handle single-ended
 	     * autoscaling, too: */
@@ -335,7 +324,8 @@ axis_checked_extend_empty_range(axis,mesg)
 		axis_array[axis].min -= widen;
 	    if (axis_array[axis].autoscale & AUTOSCALE_MAX)
 		axis_array[axis].max += widen;
-	    fprintf(stderr, "adjusting to [%g:%g]\n",
+	    if (!(axis == FIRST_Z_AXIS && !mesg)) /* set view map */
+		fprintf(stderr, "adjusting to [%g:%g]\n",
 		    axis_array[axis].min, axis_array[axis].max);
 	} else {
 	    /* user has explicitly set the range (to something empty)
@@ -349,13 +339,11 @@ axis_checked_extend_empty_range(axis,mesg)
     }
 }
 
-/*}}} */
+/* }}} */
 
-/* make smalltics for time-axis */
+/* {{{ make smalltics for time-axis */
 static double
-make_auto_time_minitics(tlevel, incr)
-    t_timelevel tlevel;
-    double incr;
+make_auto_time_minitics(t_timelevel tlevel, double incr)
 {
     double tinc = 0.0;
 
@@ -448,8 +436,9 @@ make_auto_time_minitics(tlevel, incr)
     }
     return (tinc);
 }
+/* }}} */
 
-/*{{{  copy_or_invent_formatstring() */
+/* {{{ copy_or_invent_formatstring() */
 /* Either copies the axis formatstring over to the ticfmt[] array, or
  * in case that's not applicable because the format hasn't been
  * specified correctly, invents a time/date output format by looking
@@ -457,8 +446,7 @@ make_auto_time_minitics(tlevel, incr)
  * change across the range to be unimportant */
 /* HBB 20010803: removed two arguments, and renamed function */
 char *
-copy_or_invent_formatstring(axis)
-     AXIS_INDEX axis;
+copy_or_invent_formatstring(AXIS_INDEX axis)
 {
     struct tm t_min, t_max;
 
@@ -525,16 +513,14 @@ copy_or_invent_formatstring(axis)
     return ticfmt[axis];
 }
 
-/*}}} */
+/* }}} */
 
-/*{{{  dbl_raise() used by quantize_normal_tics */
+/* {{{ dbl_raise() used by quantize_normal_tics */
 /* FIXME HBB 20000426: is this really useful? */
 static double
-dbl_raise(x, y)
-double x;
-int y;
+dbl_raise(double x, int y)
 {
-    register int i = abs(y);
+    int i = abs(y);
     double val = 1.0;
 
     while (--i >= 0)
@@ -545,9 +531,9 @@ int y;
     return (val);
 }
 
-/*}}} */
+/* }}} */
 
-/*{{{  quantize_normal_tics() */
+/* {{{ quantize_normal_tics() */
 /* the guide parameter was intended to allow the number of tics
  * to depend on the relative sizes of the plot and the font.
  * It is the approximate upper limit on number of tics allowed.
@@ -561,12 +547,10 @@ int y;
  * log10(value).  Done to allow changing the calculation method
  * to avoid numerical problems */
 double
-quantize_normal_tics(arg, guide)
-     double arg;
-     int guide;
+quantize_normal_tics(double arg, int guide)
 {
     /* order of magnitude of argument: */
-    double power = dbl_raise(10.0, floor(log10(arg))); 
+    double power = dbl_raise(10.0, floor(log10(arg)));
     double xnorm = arg / power;	/* approx number of decades */
     /* we expect 1 <= xnorm <= 10 */
     double posns = guide / xnorm; /* approx number of tic posns per decade */
@@ -604,18 +588,16 @@ quantize_normal_tics(arg, guide)
     return (tics * power);
 }
 
-/*}}} */
+/* }}} */
 
-/*{{{  make_tics() */
+/* {{{ make_tics() */
 /* Implement TIC_COMPUTED case, i.e. automatically choose a usable
  * ticking interval for the given axis. For the meaning of the guide
  * parameter, see the comment on quantize_normal_tics() */
 static double
-make_tics(axis, guide)
-    AXIS_INDEX axis;
-    int guide;
+make_tics(AXIS_INDEX axis, int guide)
 {
-    register double xr, tic;
+    double xr, tic;
 
     xr = fabs(axis_array[axis].min - axis_array[axis].max);
     tic = quantize_normal_tics(xr, guide);
@@ -629,19 +611,18 @@ make_tics(axis, guide)
     else
 	return tic;
 }
+/* }}} */
 
-/*{{{ quantize_duodecimal_tics */
+/* {{{ quantize_duodecimal_tics */
 /* HBB 20020220: New function, to be used to properly tic axes with a
  * duodecimal reference, as used in times (60 seconds, 60 minuts, 24
  * hours, 12 months). Derived from quantize_normal_tics(). The default
  * guide is assumed to be 12, here, not 20 */
 static double
-quantize_duodecimal_tics(arg, guide)
-     double arg;
-     int guide;
+quantize_duodecimal_tics(double arg, int guide)
 {
     /* order of magnitude of argument: */
-    double power = dbl_raise(12.0, floor(log(arg)/log(12.0))); 
+    double power = dbl_raise(12.0, floor(log(arg)/log(12.0)));
     double xnorm = arg / power;	/* approx number of decades */
     double posns = guide / xnorm; /* approx number of tic posns per decade */
 
@@ -672,8 +653,9 @@ quantize_duodecimal_tics(arg, guide)
 	 */
 	return power * ceil(xnorm);
 }
-/*}}} */
+/* }}} */
 
+/* {{{ quantize_time_tics */
 /* HBB 20010831: newly isolated subfunction. Used to be part of
  * make_tics() */
 /* Look at the tic interval given, and round it to a nice figure
@@ -682,10 +664,7 @@ quantize_duodecimal_tics(arg, guide)
  * this routine also modifies the static timelevel[axis] to indicate
  * the units these tics are calculated in. */
 static double
-quantize_time_tics(axis, tic, xr, guide)
-    AXIS_INDEX axis;
-    double tic, xr;
-    int guide;
+quantize_time_tics(AXIS_INDEX axis, double tic, double xr, int guide)
 {
     int guide12 = guide * 3 / 5; /* --> 12 for default of 20 */
 
@@ -735,45 +714,48 @@ quantize_time_tics(axis, tic, xr, guide)
     return (tic);
 }
 
-/*}}} */
+/* }}} */
 
+
+/* {{{ round_outward */
 /* HBB 20011204: new function (repeated code ripped out of setup_tics)
  * that rounds an axis endpoint outward. If the axis is a time/date
  * one, take care to round towards the next whole time unit, not just
  * a multiple of the (averaged) tic size */
 static double
-round_outward(axis, upwards, input)
-    AXIS_INDEX axis;		/* Axis to work on */
-    TBOOLEAN upwards;		/* extend upwards or downwards? */
-    double input;		/* the current endpoint */
+round_outward(
+    AXIS_INDEX axis,		/* Axis to work on */
+    TBOOLEAN upwards,		/* extend upwards or downwards? */
+    double input)		/* the current endpoint */
 {
     double tic = ticstep[axis];
     double result = tic * (upwards
 			   ? ceil(input / tic)
 			   : floor(input / tic));
-    
+
     if (axis_array[axis].is_timedata) {
 	double ontime = time_tic_just(timelevel[axis], result);
-	
+
 	/* FIXME: how certain is it that we don't want to *always*
 	 * return 'ontime'? */
 	if ((upwards && (ontime > result))
 	    || (!upwards && (ontime <result)))
 	    return ontime;
     }
-    
+
     return result;
 }
 
+/* }}} */
+
+/* {{{ setup_tics */
 /* setup_tics allows max number of tics to be specified but users dont
  * like it to change with size and font, so we use value of 20, which
  * is 3.5 behaviour.  Note also that if format is '', yticlin = 0, so
  * this gives division by zero.  */
 
 void
-setup_tics(axis, max)
-    AXIS_INDEX axis;
-    int max;			/* approx max number of slots available */
+setup_tics(AXIS_INDEX axis, int max)
 {
     double tic = 0;
     AXIS *this = axis_array + axis;
@@ -795,13 +777,15 @@ setup_tics(axis, max)
 
     if (ticdef->type == TIC_SERIES) {
 	ticstep[axis] = tic = ticdef->def.series.incr;
-	autoextend_min &= (ticdef->def.series.start == -VERYLARGE);
-	autoextend_max &= (ticdef->def.series.end == VERYLARGE);
+	autoextend_min = autoextend_min
+	                 && (ticdef->def.series.start == -VERYLARGE);
+	autoextend_max = autoextend_max
+	                 && (ticdef->def.series.end == VERYLARGE);
     } else if (ticdef->type == TIC_COMPUTED) {
 	ticstep[axis] = tic = make_tics(axis, max);
     } else {
 	/* user-defined, day or month */
-	autoextend_min = autoextend_max = FALSE; 
+	autoextend_min = autoextend_max = FALSE;
     }
 
     /* BUGFIX HBB 20010831: for time/date axes, if an explicit
@@ -814,10 +798,10 @@ setup_tics(axis, max)
     if (this->is_timedata && ticdef->type == TIC_SERIES)
 	quantize_time_tics(axis, tic, fabs(this->max - this->min), 20);
 
-    if (autoextend_min) 
+    if (autoextend_min)
 	this->min = round_outward(axis, ! (this->min < this->max), this->min);
 
-    if (autoextend_max) 
+    if (autoextend_max)
 	this->max = round_outward(axis, this->min < this->max, this->max);
 
 
@@ -827,37 +811,36 @@ setup_tics(axis, max)
     copy_or_invent_formatstring(axis);
 }
 
-/*{{{  gen_tics */
+/* }}} */
+
+/* {{{  gen_tics */
 /* uses global arrays ticstep[], ticfmt[], axis_array[], 
  * we use any of GRID_X/Y/X2/Y2 and  _MX/_MX2/etc - caller is expected
  * to clear the irrelevent fields from global grid bitmask
  * note this is also called from graph3d, so we need GRID_Z too
  */
 void
-gen_tics(axis, callback)
-     AXIS_INDEX axis;		/* FIRST_X_AXIS, etc */
-     tic_callback callback;	/* fn to call to actually do the work */
+gen_tics(AXIS_INDEX axis, tic_callback callback)
 {
     /* separate main-tic part of grid */
     struct lp_style_type lgrd, mgrd;
     /* tic defn */
-    struct ticdef *def = &axis_array[axis].ticdef; 
+    struct ticdef *def = &axis_array[axis].ticdef;
     /* minitics - off/default/auto/explicit */
     int minitics = axis_array[axis].minitics;
     /* minitic frequency */
     double minifreq = axis_array[axis].mtic_freq;
-     
+
 
     memcpy(&lgrd, &grid_lp, sizeof(grid_lp));
     memcpy(&mgrd, &mgrid_lp, sizeof(mgrid_lp));
     if (! axis_array[axis].gridmajor)
-	lgrd.l_type = L_TYPE_NODRAW;
+	lgrd.l_type = LT_NODRAW;
     if (! axis_array[axis].gridminor)
-	mgrd.l_type = L_TYPE_NODRAW;
-    
+	mgrd.l_type = LT_NODRAW;
 
-    if (def->type == TIC_USER) {	/* special case */
-	/*{{{  do user tics then return */
+
+    if (def->def.user) {	/* user-defined tic entries */
 	struct ticmark *mark = def->def.user;
 	double uncertain = (axis_array[axis].max - axis_array[axis].min) / 10;
 	double internal_min = axis_array[axis].min - SIGNIF * uncertain;
@@ -888,11 +871,10 @@ gen_tics(axis, callback)
 	    else
 		gprintf(label, sizeof(label), mark->label ? mark->label : ticfmt[axis], log10_base, mark->position);
 	    /* use NULL instead of label for minitic */
-	    (*callback) (axis, internal, mark->level?NULL:label, lgrd);
+	    (*callback) (axis, internal, mark->level?NULL:label, mark->level?mgrd:lgrd);
 	}
-
-	return;
-	/*}}} */
+	if (def->type == TIC_USER)
+	    return;
     }
 
     /* series-tics
@@ -926,7 +908,7 @@ gen_tics(axis, callback)
 	    lmin = lmax;
 	    lmax = temp;
 	}
-	/*{{{  choose start, step and end */
+	/* {{{  choose start, step and end */
 	switch (def->type) {
 	case TIC_SERIES:
 	    if (axis_array[axis].log) {
@@ -976,9 +958,9 @@ gen_tics(axis, callback)
 	    graph_error("Internal error : unknown tic type");
 	    return;		/* avoid gcc -Wall warning about start */
 	}
-	/*}}} */
+	/* }}} */
 
-	/*{{{  ensure ascending order */
+	/* {{{  ensure ascending order */
 	if (end < start) {
 	    double temp;
 	    temp = end;
@@ -986,32 +968,34 @@ gen_tics(axis, callback)
 	    start = temp;
 	}
 	step = fabs(step);
-	/*}}} */
+	/* }}} */
 
 	if (minitics) {
-	    /*{{{  figure out ministart, ministep, miniend */
+	    /* {{{  figure out ministart, ministep, miniend */
 	    if (minitics == MINI_USER) {
 		/* they have said what they want */
 		if (minifreq <= 0)
 		    minitics = 0;	/* not much else we can do */
-/* 		else if (axis_array[axis].log) { */
-/* 		    ministart = ministep = step / minifreq * axis_array[axis].base; */
-/* 		    miniend = step * base_array[axis]; */
-/* 		} else { */
-		ministart = ministep = step / minifreq;
-		miniend = step;
-/* 		} */
+ 		else if (axis_array[axis].log) {
+		    /* Sep 2005 - This case has been commented out since v3.7 */
+		    /* but in fact it seems correct, and fixes but #1223149 */
+ 		    ministart = ministep = step / minifreq * axis_array[axis].base;
+ 		    miniend = step * axis_array[axis].base;
+ 		} else {
+		    ministart = ministep = step / minifreq;
+		    miniend = step;
+ 		}
 	    } else if (axis_array[axis].log) {
 		if (step > 1.5) {	/* beware rounding errors */
-		    /*{{{  10,100,1000 case */
+		    /* {{{  10,100,1000 case */
 		    /* no more than five minitics */
 		    ministart = ministep = (int) (0.2 * step);
 		    if (ministep < 1)
 			ministart = ministep = 1;
 		    miniend = step;
-		    /*}}} */
+		    /* }}} */
 		} else {
-		    /*{{{  2,5,8 case */
+		    /* {{{  2,5,8 case */
 		    miniend = axis_array[axis].base;
 		    if (end - start >= 10)
 			minitics = 0;	/* none */
@@ -1022,7 +1006,7 @@ gen_tics(axis, callback)
 			ministart = 2;
 			ministep = 1;
 		    }
-		    /*}}} */
+		    /* }}} */
 		}
 	    } else if (axis_array[axis].is_timedata) {
 		ministart = ministep =
@@ -1039,10 +1023,10 @@ gen_tics(axis, callback)
 
 	    if (ministep <= 0)
 		minitics = 0;	/* dont get stuck in infinite loop */
-	    /*}}} */
+	    /* }}} */
 	}
 
-	/*{{{  a few tweaks and checks */
+	/* {{{  a few tweaks and checks */
 	/* watch rounding errors */
 	end += SIGNIF * step;
 	/* HBB 20011002: adjusting the endpoints doesn't make sense if
@@ -1058,7 +1042,7 @@ gen_tics(axis, callback)
 
 	if (step == 0)
 	    return;		/* just quietly ignore them ! */
-	/*}}} */
+	/* }}} */
 
 	/* FIXME HBB 20010121: keeping adding 'step' to 'tic' is
 	 * begging for rounding errors to strike us. */
@@ -1084,7 +1068,7 @@ gen_tics(axis, callback)
 	    } else
 		anyticput = 1;
 
-	    /*{{{  calc internal and user co-ords */
+	    /* {{{  calc internal and user co-ords */
 	    if (!axis_array[axis].log) {
 		internal = (axis_array[axis].is_timedata)
 		    ? time_tic_just(timelevel[axis], tic)
@@ -1095,14 +1079,14 @@ gen_tics(axis, callback)
 		internal = tic;
 		user = AXIS_UNDO_LOG(axis, internal);
 	    }
-	    /*}}} */
+	    /* }}} */
 	    if (internal > internal_max)
 		break;		/* gone too far - end of series = VERYLARGE perhaps */
 	    if (internal >= internal_min) {
 #if 0 /* maybe minitics!!!. user series starts below min ? */
 		continue;
 #endif
-		/*{{{  draw tick via callback */
+		/* {{{  draw tick via callback */
 		switch (def->type) {
 		case TIC_DAY:{
 			int d = (long) floor(user + 0.5) % 7;
@@ -1143,11 +1127,11 @@ gen_tics(axis, callback)
 			(*callback) (axis, internal, label, lgrd);
 		    }
 		}
-		/*}}} */
+		/* }}} */
 
 	    }
 	    if (minitics) {
-		/*{{{  process minitics */
+		/* {{{  process minitics */
 		double mplace, mtic;
 		for (mplace = ministart; mplace < miniend; mplace += ministep) {
 		    if (axis_array[axis].is_timedata)
@@ -1162,19 +1146,18 @@ gen_tics(axis, callback)
 			&& inrange(mtic, start - step * SIGNIF, end + step * SIGNIF))
 			(*callback) (axis, mtic, NULL, mgrd);
 		}
-		/*}}} */
+		/* }}} */
 	    }
 	}
     }
 }
 
-/*}}} */
+/* }}} */
 
+/* {{{ time_tic_just() */
 /* justify ticplace to a proper date-time value */
 static double
-time_tic_just(level, ticplace)
-     t_timelevel level;
-     double ticplace;
+time_tic_just(t_timelevel level, double ticplace)
 {
     struct tm tm;
 
@@ -1211,25 +1194,27 @@ time_tic_just(level, ticplace)
 	}
 	tm.tm_mday = 1;
     }
-    
+
     ticplace = gtimegm(&tm);
     return (ticplace);
 }
+/* }}} */
 
+
+/* {{{ axis_output_tics() */
 /* HBB 20000416: new routine. Code like this appeared 4 times, once
  * per 2D axis, in graphics.c. Always slightly different, of course,
  * but generally, it's always the same. I distinguish two coordinate
  * directions, here. One is the direction of the axis itself (the one
  * it's "running" along). I refer to the one orthogonal to it as
  * "non-running", below. */
-
 void
-axis_output_tics(axis, ticlabel_position, zeroaxis_basis, callback)
-     AXIS_INDEX axis;		/* axis number we're dealing with */
-     int *ticlabel_position;	/* 'non-running' coordinate */
-     AXIS_INDEX zeroaxis_basis;	/* axis to base 'non-running' position of
+axis_output_tics(
+     AXIS_INDEX axis,		/* axis number we're dealing with */
+     int *ticlabel_position,	/* 'non-running' coordinate */
+     AXIS_INDEX zeroaxis_basis,	/* axis to base 'non-running' position of
 				 * zeroaxis on */
-     tic_callback callback;	/* tic-drawing callback function */
+     tic_callback callback)	/* tic-drawing callback function */
 {
     struct termentry *t = term;
     TBOOLEAN axis_is_vertical = ((axis % SECOND_AXES) == FIRST_Y_AXIS);
@@ -1243,12 +1228,12 @@ axis_output_tics(axis, ticlabel_position, zeroaxis_basis, callback)
     } else {
 	axis_position = axis_array[zeroaxis_basis].term_lower;
 	mirror_position = axis_array[zeroaxis_basis].term_upper;
-    }	
+    }
 
     if (axis_array[axis].ticmode) {
 	/* set the globals needed by the _callback() function */
 
-	if (axis_array[axis].tic_rotate == TEXT_VERTICAL 
+	if (axis_array[axis].tic_rotate == TEXT_VERTICAL
 	    && (*t->text_angle)(TEXT_VERTICAL)) {
 	    tic_hjust = axis_is_vertical
 		? CENTRE
@@ -1300,26 +1285,33 @@ axis_output_tics(axis, ticlabel_position, zeroaxis_basis, callback)
 	    tic_direction = axis_is_second ? 1 : -1;
 	    if (axis_array[axis].ticmode & TICS_MIRROR)
 		tic_mirror = tic_start;
-	    /* put text at boundary if axis is close to boundary */
+	    /* put text at boundary if axis is close to boundary and the
+	     * corresponding boundary is switched on */
 	    if (axis_is_vertical) {
-		tic_text = (((axis_is_second ? -1 : 1)
-			     *(tic_start - axis_position) > (3 * t->h_char))
-			    ? tic_start
-			    : axis_position
-		    ) + (axis_is_second ? 1 : -1) * t->h_char;
+		if (((axis_is_second ? -1 : 1) * (tic_start - axis_position)
+		     > (3 * t->h_char))
+		    || (!axis_is_second && (!(draw_border & 2)))
+		    || (axis_is_second && (!(draw_border & 8))))
+		    tic_text = tic_start;
+		else
+		    tic_text = axis_position;
+		tic_text += (axis_is_second ? 1 : -1) * t->h_char;
 	    } else {
-		tic_text = (((axis_is_second ? -1 : 1)
-			     *(tic_start - axis_position) > (2 * t->v_char))
-			    ? tic_start + (axis_is_second
-					   ? 0
-					   : - ticscale * t->v_tic)
-			    : axis_position
-		    ) - t->v_char;
+		if (((axis_is_second ? -1 : 1) * (tic_start - axis_position)
+		     > (2 * t->v_char))
+		    || (!axis_is_second && (!(draw_border & 1)))
+		    || (axis_is_second && (!(draw_border & 4))))
+		    tic_text = tic_start +
+			(axis_is_second ? 0
+			 : - axis_array[axis].ticscale * t->v_tic);
+		else
+		    tic_text = axis_position;
+		tic_text -= t->v_char;
 	    }
 	} else {
 	    /* tics not on axis --> on border */
 	    tic_start = axis_position;
-	    tic_direction = (tic_in ? 1 : -1) * (axis_is_second ? -1 : 1);
+	    tic_direction = (axis_array[axis].tic_in ? 1 : -1) * (axis_is_second ? -1 : 1);
 	    tic_text = (*ticlabel_position);
 	}
 	/* go for it */
@@ -1327,29 +1319,34 @@ axis_output_tics(axis, ticlabel_position, zeroaxis_basis, callback)
 	(*t->text_angle) (0);	/* reset rotation angle */
     }
 }
+
+/* }}} */
 		 
+/* {{{ axis_set_graphical_range() */
+
 void
-axis_set_graphical_range(axis, lower, upper)
-    AXIS_INDEX axis;
-    unsigned int lower, upper;
+axis_set_graphical_range(AXIS_INDEX axis, unsigned int lower, unsigned int upper)
 {
     axis_array[axis].term_lower = lower;
     axis_array[axis].term_upper = upper;
 }
+/* }}} */
 
-static TBOOLEAN 
-axis_position_zeroaxis(axis)
-     AXIS_INDEX axis;
+
+/* {{{ axis_position_zeroaxis */
+static TBOOLEAN
+axis_position_zeroaxis(AXIS_INDEX axis)
 {
     TBOOLEAN is_inside = FALSE;
     AXIS *this = axis_array + axis;
 
     /* HBB 20020215: correctly treat reversed axes, too! */
-    if ((this->min >= 0.0 && this->max >= 0.0)
+    /* EAM Sep 2005: Nothing wrong with 0 at extreme of the range */
+    if ((this->min > 0.0 && this->max > 0.0)
 	|| this->log) {
 	this->term_zero = (this->max < this->min)
-	    ? this->term_upper : this->term_lower;	
-    } else if (this->min <= 0.0 && this->max <= 0.0) {
+	    ? this->term_upper : this->term_lower;
+    } else if (this->min < 0.0 && this->max < 0.0) {
 	this->term_zero = (this->max < this->min)
 	    ? this->term_lower : this->term_upper;
     } else {
@@ -1359,15 +1356,17 @@ axis_position_zeroaxis(axis)
 
     return is_inside;
 }
+/* }}} */
 
+
+/* {{{ axis_draw_2d_zeroaxis() */
 void
-axis_draw_2d_zeroaxis(axis, crossaxis)
-    AXIS_INDEX axis, crossaxis;
+axis_draw_2d_zeroaxis(AXIS_INDEX axis, AXIS_INDEX crossaxis)
 {
     AXIS *this = axis_array + axis;
 
     if (axis_position_zeroaxis(crossaxis)
-	    && (this->zeroaxis.l_type > L_TYPE_NODRAW)) {
+	    && (this->zeroaxis.l_type > LT_NODRAW)) {
 	term_apply_lp_properties(&this->zeroaxis);
 	if ((axis % SECOND_AXES) == FIRST_X_AXIS) {
 	    (*term->move) (this->term_lower, axis_array[crossaxis].term_zero);
@@ -1378,16 +1377,14 @@ axis_draw_2d_zeroaxis(axis, crossaxis)
 	}
     }
 }
+/* }}} */
 
 
-
+/* {{{ load_range() */
 /* loads a range specification from the input line into variables 'a'
  * and 'b' */
 t_autoscale
-load_range(axis, a, b, autoscale)
-     AXIS_INDEX axis;
-     double *a, *b;
-     t_autoscale autoscale;
+load_range(AXIS_INDEX axis, double *a, double *b, t_autoscale autoscale)
 {
     if (equals(c_token, "]"))
 	return (autoscale);
@@ -1423,26 +1420,27 @@ load_range(axis, a, b, autoscale)
     /* HBB 20040315: ... and clear it automatically if a fixed range
      * was given the "right" way round! */
     if ((autoscale & AUTOSCALE_BOTH) == AUTOSCALE_NONE) {
-      if (*b < *a) 
+      if (*b < *a) {
+	double temp = *a;
+
+	*a = *b; *b = temp;
 	axis_array[axis].range_flags |= RANGE_REVERSE;
-      else 
+      } else 
 	axis_array[axis].range_flags &= ~RANGE_REVERSE;
     }
 
     return (autoscale);
 }
 
+/* }}} */
+
 
 /* we determine length of the widest tick label by getting gen_ticks to
  * call this routine with every label
  */
 
 void
-widest_tic_callback(axis, place, text, grid)
-    AXIS_INDEX axis;
-    double place;
-    char *text;
-    struct lp_style_type grid;
+widest_tic_callback(AXIS_INDEX axis, double place, char *text, struct lp_style_type grid)
 {
     (void) axis;		/* avoid "unused parameter" warnings */
     (void) place;
@@ -1460,41 +1458,37 @@ widest_tic_callback(axis, place, text, grid)
  * ULIG *
  */
 
-double 
-get_writeback_min(axis)
-    AXIS_INDEX axis;
+double
+get_writeback_min(AXIS_INDEX axis)
 {
     /* printf("get min(%d)=%g\n",axis,axis_array[axis].writeback_min); */
     return axis_array[axis].writeback_min;
 }
 
-double 
-get_writeback_max(axis)
-    AXIS_INDEX axis;
+double
+get_writeback_max(AXIS_INDEX axis)
 {
     /* printf("get max(%d)=%g\n",axis,axis_array[axis].writeback_min); */
     return axis_array[axis].writeback_max;
 }
 
-void 
-set_writeback_min(axis)
-    AXIS_INDEX axis;
+void
+set_writeback_min(AXIS_INDEX axis)
 {
     double val = AXIS_DE_LOG_VALUE(axis,axis_array[axis].min);
     /* printf("set min(%d)=%g\n",axis,val); */
     axis_array[axis].writeback_min = val;
 }
 
-void 
-set_writeback_max(axis)
-    AXIS_INDEX axis;
+void
+set_writeback_max(AXIS_INDEX axis)
 {
     double val = AXIS_DE_LOG_VALUE(axis,axis_array[axis].max);
     /* printf("set max(%d)=%g\n",axis,val); */
     axis_array[axis].writeback_max = val;
 }
 
-TBOOLEAN 
+TBOOLEAN
 some_grid_selected()
 {
     AXIS_INDEX i;
@@ -1507,8 +1501,6 @@ some_grid_selected()
     return FALSE;
 }
 
-
-#ifdef PM3D
 /*
    Check and set the cb-range for use by pm3d or other palette using styles.
    Return 0 on wrong range, otherwise 1.
@@ -1534,10 +1526,12 @@ set_cbminmax()
     }
     CB_AXIS.max = axis_log_value_checked(COLOR_AXIS, CB_AXIS.max, "color axis");
 
+#if 0
     if (CB_AXIS.min == CB_AXIS.max) {
 	int_error(NO_CARET, "cannot display empty color axis range");
 	return 0;
     }
+#endif
     if (CB_AXIS.min > CB_AXIS.max) {
 	/* exchange min and max values */
 	double tmp = CB_AXIS.max;
@@ -1551,12 +1545,8 @@ set_cbminmax()
     return 1;
 }
 
-#endif /* PM3D */
-
 static void
-get_position_type(type, axes)
-enum position_type *type;
-int *axes;
+get_position_type(enum position_type *type, int *axes)
 {
     if (almost_equals(c_token, "fir$st")) {
 	++c_token;
@@ -1570,6 +1560,9 @@ int *axes;
     } else if (almost_equals(c_token, "sc$reen")) {
 	++c_token;
 	*type = screen;
+    } else if (almost_equals(c_token, "char$acter")) {
+	++c_token;
+	*type = character;
     }
     switch (*type) {
     case first_axes:
@@ -1587,21 +1580,33 @@ int *axes;
 /* get_position() - reads a position for label,arrow,key,... */
 
 void
-get_position(pos)
-struct position *pos;
+get_position(struct position *pos)
+{
+    get_position_default(pos,first_axes);
+}
+
+/* get_position() - reads a position for label,arrow,key,... 
+ * with given default coordinate system
+ */
+void
+get_position_default(struct position *pos, enum position_type default_type)
 {
     int axes;
-    enum position_type type = first_axes;
+    enum position_type type = default_type;
 
     get_position_type(&type, &axes);
     pos->scalex = type;
     GET_NUMBER_OR_TIME(pos->x, axes, FIRST_X_AXIS);
-    if (!equals(c_token, ","))
-	int_error(c_token, "expected comma");
-    ++c_token;
-    get_position_type(&type, &axes);
-    pos->scaley = type;
-    GET_NUMBER_OR_TIME(pos->y, axes, FIRST_Y_AXIS);
+
+    if (equals(c_token, ",")) {
+	++c_token;
+	get_position_type(&type, &axes);
+	pos->scaley = type;
+	GET_NUMBER_OR_TIME(pos->y, axes, FIRST_Y_AXIS);
+    } else {
+	pos->y = 0;
+	pos->scaley = type;
+    }
 
     /* z is not really allowed for a screen co-ordinate, but keep it simple ! */
     if (equals(c_token, ",")) {
@@ -1614,3 +1619,61 @@ struct position *pos;
 	pos->scalez = type;	/* same as y */
     }
 }
+
+/*
+ * Add a single tic mark, with label, to the list for this axis.
+ * To avoid duplications and overprints, sort the list and allow
+ * only one label per position.
+ * EAM - called from set.c during `set xtics`
+ *       called from datafile.c during `plot using ::xtic()`
+ */
+void
+add_tic_user(AXIS_INDEX axis, char *label, double position, int level)
+{
+    struct ticmark *tic, *newtic;
+    struct ticmark listhead;
+
+    /* Mark this axis as user-generated ticmarks only, unless the */
+    /* mix flag indicates that both user- and auto- tics are OK.  */
+    if (!axis_array[axis].ticdef.def.mix)
+	axis_array[axis].ticdef.type = TIC_USER;
+
+    /* Walk along list to sorted positional order */
+    listhead.next = axis_array[axis].ticdef.def.user;
+    listhead.position = -DBL_MAX;
+    for (tic = &listhead;
+	 tic->next && (position > tic->next->position);
+	 tic = tic->next) {
+    }
+
+    if ((tic->next == NULL) || (position < tic->next->position)) {
+	/* Make a new ticmark */
+	newtic = (struct ticmark *) gp_alloc(sizeof(struct ticmark), (char *) NULL);
+	newtic->position = position;
+	newtic->level = level;
+	/* Insert it in the list */
+	newtic->next = tic->next;
+	tic->next = newtic;
+    } else {
+	/* The new tic must duplicate position of tic->next */
+	if (position != tic->next->position)
+	    fprintf(stderr,"add_tic_user: list sort error\n");
+	newtic = tic->next;
+	/* Don't over-write a major tic with a minor tic */
+	if (newtic->level < level)
+	    return;
+	if (newtic->label) {
+	    free(newtic->label);
+	    newtic->label = NULL;
+	}
+    }
+
+    if (label)
+	newtic->label = gp_strdup(label);
+    else
+	newtic->label = NULL;
+
+    /* Make sure the listhead is kept */
+    axis_array[axis].ticdef.def.user = listhead.next;
+}
+
