@@ -1,5 +1,5 @@
 #ifndef lint
-static char *RCSid() { return RCSid("$Id: color.c,v 1.43 2003/12/14 23:23:04 broeker Exp $"); }
+static char *RCSid() { return RCSid("$Id: color.c,v 1.70.2.2 2007/02/25 13:04:50 mikulik Exp $"); }
 #endif
 
 /* GNUPLOT - color.c */
@@ -8,14 +8,12 @@ static char *RCSid() { return RCSid("$Id: color.c,v 1.43 2003/12/14 23:23:04 bro
  *
  * Petr Mikulik, since December 1998
  * Copyright: open source as much as possible
- * 
+ *
  * What is here:
  *   - Global variables declared in .h are initialized here
  *   - Palette routines
  *   - Colour box drawing
  *
- * This file is used only if PM3D is defined.
- *
 ]*/
 
 
@@ -23,8 +21,6 @@ static char *RCSid() { return RCSid("$Id: color.c,v 1.43 2003/12/14 23:23:04 bro
 #include "config.h"
 #endif
 
-#ifdef PM3D
-
 #include "color.h"
 #include "getcolor.h"
 
@@ -40,10 +36,18 @@ static char *RCSid() { return RCSid("$Id: color.c,v 1.43 2003/12/14 23:23:04 bro
 #include "alloc.h"
 
 /* COLOUR MODES - GLOBAL VARIABLES */
+
 t_sm_palette sm_palette;  /* initialized in init_color() */
 
-/* SMOOTH COLOUR BOX - GLOBAL VARIABLES */
-color_box_struct color_box; /* initialized in init_color() */
+/* Copy of palette previously in use.
+ * Exported so that change_term() can invalidate contents
+ * FIXME: better naming 
+ */
+static t_sm_palette prev_palette = {
+	-1, -1, -1, -1, -1, -1, -1, -1,
+	(rgb_color *) 0, -1
+    };
+
 
 #ifdef EXTENDED_COLOR_SPECS
 int supply_extended_color_specs = 0;
@@ -85,14 +89,7 @@ init_color()
   sm_palette.gamma = 1.5;
 
   /* initialisation of smooth color box */
-  color_box.where = SMCOLOR_BOX_DEFAULT;
-  color_box.rotation = 'v';
-  color_box.border = 1;
-  color_box.border_lt_tag = -1;
-  color_box.xorigin = 0.9;
-  color_box.yorigin = 0.2;
-  color_box.xsize = 0.1;
-  color_box.ysize = 0.63;
+  color_box = default_color_box;
 }
 
 
@@ -101,22 +98,11 @@ init_color()
    Put number of allocated colours into sm_palette.colors
  */
 int
-make_palette(void)
+make_palette()
 {
     int i;
     double gray;
 
-    /* this is simpy for deciding, if we print
-     * a message after allocating new colors */
-    static t_sm_palette save_pal = {
-	-1, -1, -1, -1, -1, -1, -1, -1,
-	(rgb_color *) 0, -1
-    };
-
-#if 0
-    GIF_show_current_palette();
-#endif
-
     if (!term->make_palette) {
 	fprintf(stderr, "Error: terminal \"%s\" does not support continuous colors.\n",term->name);
 	return 1;
@@ -129,7 +115,12 @@ make_palette(void)
 	   It will not change palette passed below, but non-NULL has to be
 	   passed there to create the header or force its initialization
 	 */
-	term->make_palette(&sm_palette);
+	if (memcmp(&prev_palette, &sm_palette, sizeof(t_sm_palette))) {
+	    term->make_palette(&sm_palette);
+	    prev_palette = sm_palette;
+	    FPRINTF((stderr,"make_palette: calling term->make_palette for term with ncolors == 0\n"));
+	} else
+	    FPRINTF((stderr,"make_palette: skipping duplicate palette for term with ncolors == 0\n"));
 	return 0;
     }
 
@@ -138,26 +129,26 @@ make_palette(void)
     if (sm_palette.use_maxcolors > 0 && i > sm_palette.use_maxcolors)
 	sm_palette.colors = sm_palette.use_maxcolors;
 
-    if (save_pal.colorFormulae < 0
-	|| sm_palette.colorFormulae != save_pal.colorFormulae
-	|| sm_palette.colorMode != save_pal.colorMode
-	|| sm_palette.formulaR != save_pal.formulaR
-	|| sm_palette.formulaG != save_pal.formulaG
-	|| sm_palette.formulaB != save_pal.formulaB
-	|| sm_palette.positive != save_pal.positive 
-	|| sm_palette.colors != save_pal.colors) {
+    if (prev_palette.colorFormulae < 0
+	|| sm_palette.colorFormulae != prev_palette.colorFormulae
+	|| sm_palette.colorMode != prev_palette.colorMode
+	|| sm_palette.formulaR != prev_palette.formulaR
+	|| sm_palette.formulaG != prev_palette.formulaG
+	|| sm_palette.formulaB != prev_palette.formulaB
+	|| sm_palette.positive != prev_palette.positive
+	|| sm_palette.colors != prev_palette.colors) {
 	/* print the message only if colors have changed */
 	if (interactive)
 	fprintf(stderr, "smooth palette in %s: available %i color positions; using %i of them\n", term->name, i, sm_palette.colors);
     }
 
-    save_pal = sm_palette;
+    prev_palette = sm_palette;
 
     if (sm_palette.color != NULL) {
 	free(sm_palette.color);
 	sm_palette.color = NULL;
     }
-    sm_palette.color = gp_alloc( sm_palette.colors * sizeof(rgb_color), 
+    sm_palette.color = gp_alloc( sm_palette.colors * sizeof(rgb_color),
 				 "pm3d palette color");
 
     /*  fill sm_palette.color[]  */
@@ -165,57 +156,72 @@ make_palette(void)
 	gray = (double) i / (sm_palette.colors - 1);	/* rescale to [0;1] */
 	rgb1_from_gray( gray, &(sm_palette.color[i]) );
     }
-    
+
     /* let the terminal make the palette from the supplied RGB triplets */
     term->make_palette(&sm_palette);
 
-#if 0
-    GIF_show_current_palette();
-#endif
-
     return 0;
 }
 
+/*
+ * Force a mismatch between the current palette and whatever is sent next,
+ * so that the new one will always be loaded 
+ */
+void
+invalidate_palette()
+{
+    prev_palette.colors = -1;
+}
 
 /*
    Set the colour on the terminal
    Currently, each terminal takes care of remembering the current colour,
-   so there is not much to do here---well, except for reversing the gray
-   according to sm_palette.positive == SMPAL_POSITIVE or SMPAL_NEGATIVE
+   so there is not much to do here.
  */
 void
 set_color(double gray)
 {
+    t_colorspec color;
     if (!(term->set_color))
 	return;
-    if (sm_palette.positive == SMPAL_NEGATIVE)
-	gray = 1 - gray;
-    term->set_color(gray);
+    color.type = TC_FRAC;
+    color.value = gray;
+    term->set_color(&color);
 }
 
-
-#if 0
-/*
-   Makes mapping from real 3D coordinates to 2D terminal coordinates,
-   then draws filled polygon
- */
-static void
-filled_polygon(int points, gpdPoint * corners)
+void
+set_rgbcolor(int rgblt)
 {
-    int i;
-    gpiPoint *icorners;
-    icorners = gp_alloc(points * sizeof(gpiPoint), "filled_polygon corners");
-    for (i = 0; i < points; i++)
-	map3d_xy(corners[i].x, corners[i].y, corners[i].z, &icorners[i].x, &icorners[i].y);
-#ifdef EXTENDED_COLOR_SPECS
-    if (supply_extended_color_specs) {
-	icorners[0].spec.gray = -1;	/* force solid color */
+    t_colorspec color;
+    if (!(term->set_color))
+	return;
+    color.type = TC_RGB;
+    color.lt = rgblt;
+    term->set_color(&color);
+}
+
+void ifilled_quadrangle(gpiPoint* icorners)
+{
+    icorners->style = FS_OPAQUE;
+    term->filled_polygon(4, icorners);
+
+    if (pm3d.hidden3d_tag) {
+
+	int i;
+
+	/* Colour has changed, thus must apply properties again. That's because
+	   gnuplot has no inner notion of color.
+	 */
+	struct lp_style_type lp;
+	lp_use_properties(&lp, pm3d.hidden3d_tag, 1);
+	term_apply_lp_properties(&lp);
+
+	term->move(icorners[0].x, icorners[0].y);
+	for (i = 3; i >= 0; i--) {
+	    term->vector(icorners[i].x, icorners[i].y);
+	}
     }
-#endif
-    term->filled_polygon(points, icorners);
-    free(icorners);
 }
-#endif
 
 
 /* The routine above for 4 points explicitly.
@@ -238,22 +244,7 @@ filled_quadrangle(gpdPoint * corners)
 	map3d_xy(corners[i].x, corners[i].y, corners[i].z, &icorners[i].x, &icorners[i].y);
     }
 
-    term->filled_polygon(4, icorners);
-
-    if (pm3d.hidden3d_tag) {
-
-	/* Colour has changed, thus must apply properties again. That's because
-	   gnuplot has no inner notion of color.
-	 */
-	struct lp_style_type lp;
-	lp_use_properties(&lp, pm3d.hidden3d_tag, 1);
-	term_apply_lp_properties(&lp);
-
-	term->move(icorners[0].x, icorners[0].y);
-	for (i = 3; i >= 0; i--) {
-	    term->vector(icorners[i].x, icorners[i].y);
-	}
-    }
+    ifilled_quadrangle(icorners);
 }
 
 
@@ -274,6 +265,7 @@ filled_polygon_3dcoords(int points, struct coordinate GPHUGE * coords)
 	icorners[0].spec.gray = -1;	/* force solid color */
     }
 #endif
+    icorners->style = FS_OPAQUE;
     term->filled_polygon(points, icorners);
     free(icorners);
 }
@@ -296,6 +288,7 @@ filled_polygon_3dcoords_zfixed(int points, struct coordinate GPHUGE * coords, do
 	icorners[0].spec.gray = -1;	/* force solid color */
     }
 #endif
+    icorners->style = FS_OPAQUE;
     term->filled_polygon(points, icorners);
     free(icorners);
 }
@@ -315,8 +308,7 @@ filled_polygon_3dcoords_zfixed(int points, struct coordinate GPHUGE * coords, do
    PS routine.
  */
 static void
-draw_inside_color_smooth_box_postscript(out)
-FILE * out;
+draw_inside_color_smooth_box_postscript(FILE * out)
 {
     int scale_x = (cb_x_to - cb_x_from), scale_y = (cb_y_to - cb_y_from);
     fputs("stroke gsave\t%% draw gray scale smooth box\n"
@@ -340,7 +332,7 @@ FILE * out;
     fputs("/y0 y0 ystep add def /ii ii 1 add def\n"
 	  "ii imax ge {exit} if } loop\n"
 	  "grestore 0 setgray\n", out);
-}	
+}
 
 
 
@@ -350,14 +342,13 @@ FILE * out;
    over all thin rectangles
  */
 static void
-draw_inside_color_smooth_box_bitmap(out)
-FILE * out;
+draw_inside_color_smooth_box_bitmap(FILE * out)
 {
     int steps = 128; /* I think that nobody can distinguish more colours drawn in the palette */
     int i, xy, xy2, xy_from, xy_to;
     double xy_step, gray;
     gpiPoint corners[4];
-    
+
     (void) out;			/* to avoid "unused parameter" warning */
     if (color_box.rotation == 'v') {
 	corners[0].x = corners[3].x = cb_x_from;
@@ -374,6 +365,8 @@ FILE * out;
 
     for (i = 0; i < steps; i++) {
 	gray = (double) i / steps;	/* colours equidistantly from [0,1] */
+	if (sm_palette.positive == SMPAL_NEGATIVE)
+	    gray = 1 - gray;
 	/* Set the colour (also for terminals which support extended specs). */
 	set_color(gray);
 	xy = xy_from + (int) (xy_step * i);
@@ -391,6 +384,7 @@ FILE * out;
 	}
 #endif
 	/* print the rectangle with the given colour */
+	corners->style = FS_OPAQUE;
 	term->filled_polygon(4, corners);
     }
 }
@@ -398,14 +392,14 @@ FILE * out;
 /* Notice HBB 20010720: would be static, but HP-UX gcc bug forbids
  * this, for now */
 void
-cbtick_callback(axis, place, text, grid)
-    AXIS_INDEX axis;
-    double place;
-    char *text;
-    struct lp_style_type grid; /* linetype or -2 for no grid */
+cbtick_callback(
+    AXIS_INDEX axis,
+    double place,
+    char *text,
+    struct lp_style_type grid) /* linetype or -2 for no grid */
 {
-    int len = (text ? ticscale : miniticscale)
-	* (tic_in ? -1 : 1) * (term->h_tic);
+    int len = (text ? CB_AXIS.ticscale : CB_AXIS.miniticscale)
+	* (CB_AXIS.tic_in ? -1 : 1) * (term->h_tic);
     double cb_place = (place - CB_AXIS.min) / (CB_AXIS.max - CB_AXIS.min);
 	/* relative z position along the colorbox axis */
     unsigned int x1, y1, x2, y2;
@@ -427,7 +421,7 @@ cbtick_callback(axis, place, text, grid)
     }
 
     /* draw grid line */
-    if (grid.l_type > L_TYPE_NODRAW) {
+    if (grid.l_type > LT_NODRAW) {
 	term_apply_lp_properties(&grid);	/* grid linetype */
 	if (color_box.rotation == 'h') {
 	    (*term->move) (x1, cb_y_from);
@@ -445,32 +439,33 @@ cbtick_callback(axis, place, text, grid)
 
     /* draw label */
     if (text) {
+	/* get offset */
+	int offsetx, offsety;
+	map3d_position_r(&(axis_array[axis].ticdef.offset),
+			 &offsetx, &offsety, "cbtics");
 	/* User-specified different color for the tics text */
 	if (axis_array[axis].ticdef.textcolor.lt != TC_DEFAULT)
-	    apply_textcolor(&(axis_array[axis].ticdef.textcolor), term);
+	    apply_pm3dcolor(&(axis_array[axis].ticdef.textcolor), term);
 	if (color_box.rotation == 'h') {
 	    int y3 = cb_y_from - (term->v_char);
+	    int hrotate = 0;
+
+	    if (axis_array[axis].tic_rotate
+		&& (*term->text_angle)(axis_array[axis].tic_rotate))
+		    hrotate = axis_array[axis].tic_rotate;
 	    if (len > 0) y3 -= len; /* add outer tics len */
 	    if (y3<0) y3 = 0;
-#if 1
-	    if (term->justify_text)
-		term->justify_text(CENTRE);
-	    (*term->put_text)(x2, y3, text);
-#else /* clipping does not work properly for text around 3d graph */
-	    clip_put_text_just(x2, y3, text, CENTRE, JUST_TOP, 
-			       axis_array[axis].ticdef.font);
-#endif
+	    write_multiline(x2+offsetx, y3+offsety, text,
+			    (hrotate ? LEFT : CENTRE), CENTRE, hrotate,
+			    axis_array[axis].ticdef.font);
+	    if (hrotate)
+		(*term->text_angle)(0);
 	} else {
 	    unsigned int x3 = cb_x_to + (term->h_char);
 	    if (len > 0) x3 += len; /* add outer tics len */
-#if 1
-	    if (term->justify_text)
-		term->justify_text(LEFT);
-	    (*term->put_text)(x3, y2, text);
-#else /* clipping does not work properly for text around 3d graph */
-	    clip_put_text_just(x3, y2, text, LEFT, JUST_CENTRE, 
-			       axis_array[axis].ticdef.font);
-#endif
+	    write_multiline(x3+offsetx, y2+offsety, text,
+			    LEFT, CENTRE, 0.0,
+			    axis_array[axis].ticdef.font);
 	}
 	term_apply_lp_properties(&border_lp);	/* border linetype */
     }
@@ -493,10 +488,10 @@ cbtick_callback(axis, place, text, grid)
    Finally the main colour smooth box drawing routine
  */
 void
-draw_color_smooth_box()
+draw_color_smooth_box(int plot_mode)
 {
     double tmp;
-    FILE *out = postscript_gpoutfile;	/* either gpoutfile or PSLATEX_auxfile */
+    FILE *out = gppsfile;	/* either gpoutfile or PSLATEX_auxfile */
 
     if (color_box.where == SMCOLOR_BOX_NO)
 	return;
@@ -513,52 +508,68 @@ draw_color_smooth_box()
        }
      */
     if (color_box.where == SMCOLOR_BOX_USER) {
-	cb_x_from = color_box.xorigin * (term->xmax) + 0.5;
-	cb_y_from = color_box.yorigin * (term->ymax) + 0.5;
-	cb_x_to = color_box.xsize * (term->xmax) + 0.5;
-	cb_y_to = color_box.ysize * (term->ymax) + 0.5;
+	if (!is_3d_plot) {
+	    double xtemp, ytemp;
+	    map_position(&color_box.origin, &cb_x_from, &cb_y_from, "cbox");
+	    map_position_r(&color_box.size, &xtemp, &ytemp, "cbox");
+	    cb_x_to = xtemp;
+	    cb_y_to = ytemp;
+	} else if (splot_map && is_3d_plot) {
+	    /* In map view mode we allow any coordinate system for placement */
+	    double xtemp, ytemp;
+	    map3d_position_double(&color_box.origin, &xtemp, &ytemp, "cbox");
+	    cb_x_from = xtemp;
+	    cb_y_from = ytemp;
+	    map3d_position_r(&color_box.size, &cb_x_to, &cb_y_to, "cbox");
+	} else {
+	    /* But in full 3D mode we only allow screen coordinates */
+	    cb_x_from = color_box.origin.x * (term->xmax) + 0.5;
+	    cb_y_from = color_box.origin.y * (term->ymax) + 0.5;
+	    cb_x_to = color_box.size.x * (term->xmax) + 0.5;
+	    cb_y_to = color_box.size.y * (term->ymax) + 0.5;
+	}
 	cb_x_to += cb_x_from;
 	cb_y_to += cb_y_from;
-    } else {			/* color_box.where == SMCOLOR_BOX_DEFAULT */
-#if 0 
-	/* HBB 20031215: replaced this by view-independant placment method */
-	double dx = (X_AXIS.max - X_AXIS.min);
-	/* don't use CB_AXIS here, CB_AXIS might be completely
-	 * unrelated to the Z axis 26 Jan 2002 (joze) */
-	double dz = Z_AXIS.max - Z_AXIS.min;
 
-	/* note: [0.05 0.35; 0.20 0.00] were the values before cbaxis */
-	map3d_xy(X_AXIS.max + dx * 0.03, Y_AXIS.max, base_z + dz * 0.35,
-		 &cb_x_from, &cb_y_from);
-	map3d_xy(X_AXIS.max + dx * 0.11, Y_AXIS.max, ceiling_z - dz * 0,
-		 &cb_x_to, &cb_y_to);
-	if (cb_y_from == cb_y_to || cb_x_from == cb_x_to) { /* map, i.e. plot with "set view 0,0 or 180,0" */
-	    dz = Y_AXIS.max - Y_AXIS.min;
-	    /* note: [0.04 0.25; 0.18 0.25] were the values before cbaxis */
-	    map3d_xy(X_AXIS.max + dx * 0.025, Y_AXIS.min, base_z, & cb_x_from, &cb_y_from);
+    } else { /* color_box.where == SMCOLOR_BOX_DEFAULT */
+	if (plot_mode == MODE_SPLOT && !splot_map) {
+	    /* HBB 20031215: new code.  Constants fixed to what the result
+	     * of the old code in default view (set view 60,30,1,1)
+	     * happened to be. Somebody fix them if they're not right! */
+	    cb_x_from = xmiddle + 0.709 * xscaler;
+	    cb_x_to   = xmiddle + 0.778 * xscaler;
+	    cb_y_from = ymiddle - 0.147 * yscaler;
+	    cb_y_to   = ymiddle + 0.497 * yscaler;
+
+	} else if (is_3d_plot) {
+	    /* MWS 09-Dec-05, make color box full size for splot maps. */
+	    double dx = (X_AXIS.max - X_AXIS.min);
+	    map3d_xy(X_AXIS.max + dx * 0.025, Y_AXIS.min, base_z, &cb_x_from, &cb_y_from);
 	    map3d_xy(X_AXIS.max + dx * 0.075, Y_AXIS.max, ceiling_z, &cb_x_to, &cb_y_to);
+	} else { /* 2D plot */
+	    struct position default_origin = {graph,graph,graph, 1.025, 0, 0};
+	    struct position default_size = {graph,graph,graph, 0.05, 1.0, 0};
+	    double xtemp, ytemp;
+	    map_position(&default_origin, &cb_x_from, &cb_y_from, "cbox");
+	    map_position_r(&default_size, &xtemp, &ytemp, "cbox");
+	    cb_x_to = xtemp + cb_x_from;
+	    cb_y_to = ytemp + cb_y_from;
 	}
-#else
-	/* HBB 20031215: new code.  Constants fixed to what the result
-	 * of the old code in default view (set view 60,30,1,1)
-	 * happened to be. Somebody fix them if they're not right!*/
-	cb_x_from = xmiddle + 0.709 * xscaler;
-	cb_x_to   = xmiddle + 0.778 * xscaler;
-	cb_y_from = ymiddle - 0.147 * yscaler;
-	cb_y_to   = ymiddle + 0.497 * yscaler;
-#endif
+
 	/* now corrections for outer tics */
 	if (color_box.rotation == 'v') {
-	    int len = (tic_in ? -1 : 1) * ticscale * (term->h_tic); /* positive for outer tics */
-	    if (len > 0) {
-		if (CB_AXIS.ticmode & TICS_MIRROR) {
-		    cb_x_from += len;
-		    cb_x_to += len;
-		}
-		if (axis_array[FIRST_Y_AXIS].ticmode & TICS_MIRROR) {
-		    cb_x_from += len;
-		    cb_x_to += len;
-		}
+	    int cblen = (CB_AXIS.tic_in ? -1 : 1) * CB_AXIS.ticscale * 
+		(term->h_tic); /* positive for outer tics */
+	    int ylen = (Y_AXIS.tic_in ? -1 : 1) * Y_AXIS.ticscale * 
+		(term->h_tic); /* positive for outer tics */
+	    if ((cblen > 0) && (CB_AXIS.ticmode & TICS_MIRROR)) {
+		cb_x_from += cblen;
+		cb_x_to += cblen;
+	    }
+	    if ((ylen > 0) && 
+		(axis_array[FIRST_Y_AXIS].ticmode & TICS_MIRROR)) {
+		cb_x_from += ylen;
+		cb_x_to += ylen;
 	    }
 	}
     }
@@ -572,7 +583,7 @@ draw_color_smooth_box()
     /* Optimized version of the smooth colour box in postscript. Advantage:
        only few lines of code is written into the output file.
      */
-    if (postscript_gpoutfile)
+    if (gppsfile)
 	draw_inside_color_smooth_box_postscript(out);
     else
 	draw_inside_color_smooth_box_bitmap(out);
@@ -588,11 +599,13 @@ draw_color_smooth_box()
 	    /* black solid colour should be chosen, so it's border linetype */
 	    term_apply_lp_properties(&border_lp);
 	}
+	newpath();
 	(term->move) (cb_x_from, cb_y_from);
 	(term->vector) (cb_x_to, cb_y_from);
 	(term->vector) (cb_x_to, cb_y_to);
 	(term->vector) (cb_x_from, cb_y_to);
 	(term->vector) (cb_x_from, cb_y_from);
+	closepath();
 
 	/* Set line properties to some value, this also draws lines in postscript terminals. */
 	    term_apply_lp_properties(&border_lp);
@@ -601,19 +614,22 @@ draw_color_smooth_box()
     /* draw tics */
     if (axis_array[COLOR_AXIS].ticmode) {
 	term_apply_lp_properties(&border_lp); /* border linetype */
-	setup_tics(COLOR_AXIS, 20);
 	gen_tics(COLOR_AXIS, cbtick_callback );
     }
 
     /* write the colour box label */
-    if (*CB_AXIS.label.text) {
+    if (CB_AXIS.label.text) {
 	int x, y;
-	apply_textcolor(&(CB_AXIS.label.textcolor),term);
+	apply_pm3dcolor(&(CB_AXIS.label.textcolor),term);
 	if (color_box.rotation == 'h') {
-	    int len = ticscale * (tic_in ? 1 : -1) * (term->v_tic);
-	    x = (cb_x_from + cb_x_to) / 2 + CB_AXIS.label.xoffset * term->h_char;
+	    int len = CB_AXIS.ticscale * (CB_AXIS.tic_in ? 1 : -1) * 
+		(term->v_tic);
+
+	    map3d_position_r(&(CB_AXIS.label.offset), &x, &y, "smooth_box");
+	    x += (cb_x_from + cb_x_to) / 2;
+
 #define DEFAULT_Y_DISTANCE 1.0
-	    y = cb_y_from + (CB_AXIS.label.yoffset - DEFAULT_Y_DISTANCE - 1.7) * term->v_char;
+	    y += cb_y_from + (- DEFAULT_Y_DISTANCE - 1.7) * term->v_char;
 #undef DEFAULT_Y_DISTANCE
 	    if (len < 0) y += len;
 	    if (x<0) x = 0;
@@ -621,22 +637,24 @@ draw_color_smooth_box()
 	    write_multiline(x, y, CB_AXIS.label.text, CENTRE, JUST_CENTRE, 0,
 			    CB_AXIS.label.font);
 	} else {
-	    int len = ticscale * (tic_in ? -1 : 1) * (term->h_tic);
+	    int len = CB_AXIS.ticscale * (CB_AXIS.tic_in ? -1 : 1) *
+		(term->h_tic);
 	    /* calculate max length of cb-tics labels */
 	    widest_tic_strlen = 0;
 	    if (CB_AXIS.ticmode & TICS_ON_BORDER) {
 	      	widest_tic_strlen = 0; /* reset the global variable */
 		gen_tics(COLOR_AXIS, /* 0, */ widest_tic_callback);
 	    }
+	    map3d_position_r(&(CB_AXIS.label.offset), &x, &y, "smooth_box");
 #define DEFAULT_X_DISTANCE 1.0
-	    x = cb_x_to + (CB_AXIS.label.xoffset + widest_tic_strlen + DEFAULT_X_DISTANCE + 1.5) * term->h_char;
+	    x += cb_x_to + (widest_tic_strlen + DEFAULT_X_DISTANCE + 1.5) * term->h_char;
 #undef DEFAULT_X_DISTANCE
 	    if (len > 0) x += len;
-	    y = (cb_y_from + cb_y_to) / 2 + CB_AXIS.label.yoffset * term->v_char;
+	    y += (cb_y_from + cb_y_to) / 2;
 	    if (x<0) x = 0;
 	    if (y<0) y = 0;
 	    if ((*term->text_angle)(TEXT_VERTICAL)) {
-		write_multiline(x, y, CB_AXIS.label.text, CENTRE, JUST_TOP, 
+		write_multiline(x, y, CB_AXIS.label.text, CENTRE, JUST_TOP,
 				TEXT_VERTICAL, CB_AXIS.label.font);
 		(*term->text_angle)(0);
 	    } else {
@@ -648,7 +666,3 @@ draw_color_smooth_box()
 
 }
 
-
-
-
-#endif /* PM3D */
