@@ -1,5 +1,5 @@
 #ifndef lint
-static char *RCSid() { return RCSid("$Id: eval.c,v 1.16 2004/04/13 17:23:53 broeker Exp $"); }
+static char *RCSid() { return RCSid("$Id: eval.c,v 1.51 2006/08/22 03:08:12 sfeam Exp $"); }
 #endif
 
 /* GNUPLOT - eval.c */
@@ -48,12 +48,12 @@ static char *RCSid() { return RCSid("$Id: eval.c,v 1.16 2004/04/13 17:23:53 broe
 #include "specfun.h"
 #include "standard.h"
 #include "util.h"
+#include "version.h"
 
 #include <signal.h>
 #include <setjmp.h>
 
 /* Internal prototypes */
-static char *num_to_str __PROTO((double r));
 static RETSIGTYPE fpe __PROTO((int an_int));
 #ifdef APOLLO
 static pfm_$fh_func_val_t apollo_sigfpe(pfm_$fault_rec_t & fault_rec)
@@ -61,6 +61,7 @@ static pfm_$fh_func_val_t apollo_sigfpe(pfm_$fault_rec_t & fault_rec)
 
 /* Global variables exported by this module */
 struct udvt_entry udv_pi = { NULL, "pi", FALSE, {INTGR, {0} } };
+struct udvt_entry *udv_NaN;
 /* first in linked list */
 struct udvt_entry *first_udv = &udv_pi;
 struct udft_entry *first_udf = NULL;
@@ -89,9 +90,6 @@ const struct ft_entry GPFAR ft[] =
     {"pushd1",  f_pushd1},
     {"pushd2",  f_pushd2},
     {"pushd",  f_pushd},
-#ifdef GP_ISVAR
-    {"pushv", f_pushv},
-#endif  /*GP_ISVAR*/
     {"call",  f_call},
     {"calln",  f_calln},
     {"lnot",  f_lnot},
@@ -117,6 +115,12 @@ const struct ft_entry GPFAR ft[] =
     {"factorial",  f_factorial},
     {"bool",  f_bool},
     {"dollars",  f_dollars},	/* for using extension */
+#ifdef GP_STRING_VARS
+    {"concatenate",  f_concatenate},	/* for string variables only */
+    {"eqs",  f_eqs},			/* for string variables only */
+    {"nes",  f_nes},			/* for string variables only */
+    {"[]",  f_range},			/* for string variables only */
+#endif
     {"jump",  f_jump},
     {"jumpz",  f_jumpz},
     {"jumpnz",  f_jumpnz},
@@ -157,9 +161,9 @@ const struct ft_entry GPFAR ft[] =
     {"rand",  f_rand},
     {"floor",  f_floor},
     {"ceil",  f_ceil},
-#ifdef GP_ISVAR
-    {"defined",  f_isvar},       /* isvar function */
-#endif  /*GP_ISVAR*/
+#ifdef BACKWARDS_COMPATIBLE
+    {"defined",  f_exists},       /* deprecated syntax defined(foo) */
+#endif
 
     {"norm",  f_normal},	/* XXX-JG */
     {"inverf",  f_inverse_erf},	/* XXX-JG */
@@ -182,6 +186,20 @@ const struct ft_entry GPFAR ft[] =
     {"tm_wday",  f_tmwday},	/* for timeseries */
     {"tm_yday",  f_tmyday},	/* for timeseries */
 
+#ifdef GP_STRING_VARS
+    {"stringcolumn",  f_stringcolumn},	/* for using */
+    {"sprintf",  f_sprintf},	/* for string variables only */
+    {"gprintf",  f_gprintf},	/* for string variables only */
+    {"strlen",  f_strlen},	/* for string variables only */
+    {"strstrt",  f_strstrt},	/* for string variables only */
+    {"substr",  f_range},	/* for string variables only */
+    {"word", f_words},		/* for string variables only */
+    {"words", f_words},		/* implemented as word(s,-1) */
+    {"system", f_system},       /* "dynamic backtics" */
+    {"exist", f_exists},	/* exists("foo") replaces defined(foo) */
+    {"exists", f_exists},	/* exists("foo") replaces defined(foo) */
+#endif
+
     {NULL, NULL}
 };
 
@@ -196,8 +214,7 @@ static JMP_BUF fpe_env;
 /* Internal helper functions: */
 
 static RETSIGTYPE
-fpe(an_int)
-    int an_int;
+fpe(int an_int)
 {
 #if defined(MSDOS) && !defined(__EMX__) && !defined(DJGPP) && !defined(_Windows) || defined(DOS386)
     /* thanks to lotto@wjh12.UUCP for telling us about this  */
@@ -221,9 +238,9 @@ fpe(an_int)
  * On an Apollo, the OS can signal a couple errors that are not mapped into
  * SIGFPE, namely signalling NaN and branch on an unordered comparison.  I
  * suppose there are others, but none of these are documented, so I handle
- * them as they arise. 
+ * them as they arise.
  *
- * Anyway, we need to catch these faults and signal SIGFPE. 
+ * Anyway, we need to catch these faults and signal SIGFPE.
  */
 
 static pfm_$fh_func_val_t
@@ -245,97 +262,61 @@ apollo_pfm_catch()
 }
 #endif /* APOLLO */
 
-/* Helper for disp_value(): display a single number in decimal
- * format. Rotates through 4 buffers 's[j]', and returns pointers to
- * them, to avoid execution ordering problems if this function is
- * called more than once between sequence points. */
-static char *
-num_to_str(r)
-    double r;
-{
-    static int i = 0;
-    static char s[4][25];
-    int j = i++;
-
-    if (i > 3)
-	i = 0;
-
-    sprintf(s[j], "%.15g", r);
-    if (strchr(s[j], '.') == NULL &&
-	strchr(s[j], 'e') == NULL &&
-	strchr(s[j], 'E') == NULL)
-	strcat(s[j], ".0");
-
-    return s[j];
-}
-
 /* Exported functions */
 
 /* First, some functions tha help other modules use 'struct value' ---
  * these might justify a separate module, but I'll stick with this,
  * for now */
 
-/* Display a value in human-readable form. */
-void
-disp_value(fp, val)
-    FILE *fp;
-    struct value *val;
-{
-    switch (val->type) {
-    case INTGR:
-	fprintf(fp, "%d", val->v.int_val);
-	break;
-    case CMPLX:
-	if (val->v.cmplx_val.imag != 0.0)
-	    fprintf(fp, "{%s, %s}",
-		    num_to_str(val->v.cmplx_val.real),
-		    num_to_str(val->v.cmplx_val.imag));
-	else
-	    fprintf(fp, "%s",
-		    num_to_str(val->v.cmplx_val.real));
-	break;
-    default:
-	int_error(NO_CARET, "unknown type in disp_value()");
-    }
-}
-
-
+/* returns the real part of val */
 double
-real(val)			/* returns the real part of val */
-    struct value *val;
+real(struct value *val)
 {
     switch (val->type) {
     case INTGR:
 	return ((double) val->v.int_val);
     case CMPLX:
 	return (val->v.cmplx_val.real);
+#ifdef GP_STRING_VARS
+    case STRING:              /* is this ever used? */
+	return (atof(val->v.string_val));
+#endif
+    default:
+	int_error(NO_CARET, "unknown type in real()");
     }
-    int_error(NO_CARET, "unknown type in real()");
     /* NOTREACHED */
     return ((double) 0.0);
 }
 
 
+/* returns the imag part of val */
 double
-imag(val)			/* returns the imag part of val */
-    struct value *val;
+imag(struct value *val)
 {
     switch (val->type) {
     case INTGR:
 	return (0.0);
     case CMPLX:
 	return (val->v.cmplx_val.imag);
+#ifdef GP_STRING_VARS
+    case STRING:
+	/* This is where we end up if the user tries: */
+	/*     x = 2;  plot sprintf(format,x)         */
+	int_warn(NO_CARET, "encountered a string when expecting a number");
+	int_error(NO_CARET, "Did you try to generate a file name using dummy variable x or y?");
+#endif
+    default:
+	int_error(NO_CARET, "unknown type in imag()");
     }
-    int_error(NO_CARET, "unknown type in imag()");
     /* NOTREACHED */
     return ((double) 0.0);
 }
 
 
 
+/* returns the magnitude of val */
 double
-magnitude(val)			/* returns the magnitude of val */
-    struct value *val;
+magnitude(struct value *val)
 {
     switch (val->type) {
     case INTGR:
@@ -345,17 +326,18 @@ magnitude(val)			/* returns the magnitude of val */
 		     val->v.cmplx_val.real +
 		     val->v.cmplx_val.imag *
 		     val->v.cmplx_val.imag));
+    default:
+	int_error(NO_CARET, "unknown type in magnitude()");
     }
-    int_error(NO_CARET, "unknown type in magnitude()");
     /* NOTREACHED */
     return ((double) 0.0);
 }
 
 
 
+/* returns the angle of val */
 double
-angle(val)			/* returns the angle of val */
-    struct value *val;
+angle(struct value *val)
 {
     switch (val->type) {
     case INTGR:
@@ -369,17 +351,16 @@ angle(val)			/* returns the angle of val */
 	}
 	return (atan2(val->v.cmplx_val.imag,
 		      val->v.cmplx_val.real));
+    default:
+	int_error(NO_CARET, "unknown type in angle()");
     }
-    int_error(NO_CARET, "unknown type in angle()");
     /* NOTREACHED */
     return ((double) 0.0);
 }
 
 
 struct value *
-Gcomplex(a, realpart, imagpart)
-    struct value *a;
-    double realpart, imagpart;
+Gcomplex(struct value *a, double realpart, double imagpart)
 {
     a->type = CMPLX;
     a->v.cmplx_val.real = realpart;
@@ -389,37 +370,61 @@ Gcomplex(a, realpart, imagpart)
 
 
 struct value *
-Ginteger(a, i)
-    struct value *a;
-    int i;
+Ginteger(struct value *a, int i)
 {
     a->type = INTGR;
     a->v.int_val = i;
     return (a);
 }
 
+#ifdef GP_STRING_VARS
+struct value *
+Gstring(struct value *a, char *s)
+{
+    a->type = STRING;
+    a->v.string_val = s;
+    return (a);
+}
+#endif
+
+/* It is always safe to call gpfree_string with a->type is INTGR or CMPLX.
+ * However it would be fatal to call it with a->type = STRING if a->string_val
+ * was not obtained by a previous call to gp_alloc(), or has already been freed.
+ * Thus 'a->type' is set to INTGR afterwards to make subsequent calls safe.
+ */
+struct value *
+gpfree_string(struct value *a)
+{
+#ifdef GP_STRING_VARS
+    if (a->type == STRING) {
+	free(a->v.string_val);
+	/* I would have set it to INVALID if such a type existed */
+	a->type = INTGR;
+    }
+#endif
+    return a;
+}
 
 /* some machines have trouble with exp(-x) for large x
- * if MINEXP is defined at compile time, use gp_exp(x) instead,
- * which returns 0 for exp(x) with x < MINEXP
+ * if E_MINEXP is defined at compile time, use gp_exp(x) instead,
+ * which returns 0 for exp(x) with x < E_MINEXP
  * exp(x) will already have been defined as gp_exp(x) in plot.h
  */
 
 double
-gp_exp(x)
-    double x;
+gp_exp(double x)
 {
-#ifdef MINEXP    
-    return (x < (MINEXP)) ? 0.0 : exp(x);
-#else  /* MINEXP */
+#ifdef E_MINEXP
+    return (x < (E_MINEXP)) ? 0.0 : exp(x);
+#else  /* E_MINEXP */
     int old_errno = errno;
     double result = exp(x);
-    
+
     /* exp(-large) quite uselessly raises ERANGE --- stop that */
     if (result == 0.0)
 	errno = old_errno;
     return result;
-#endif /* MINEXP */
+#endif /* E_MINEXP */
 }
 
 void
@@ -438,9 +443,14 @@ warning:  internal error--stack not empty!\n\
           (function called with too many parameters?)\n");
 }
 
+TBOOLEAN
+more_on_stack()
+{
+    return (s_p >= 0);
+}
+
 struct value *
-pop(x)
-    struct value *x;
+pop(struct value *x)
 {
     if (s_p < 0)
 	int_error(NO_CARET, "stack underflow (function call with missing parameters?)");
@@ -448,20 +458,41 @@ pop(x)
     return (x);
 }
 
+#if (GP_STRING_VARS > 1)
+/*
+ * Allow autoconversion of string variables to floats if they
+ * are dereferenced in a numeric context.
+ */
+struct value *
+pop_or_convert_from_string(struct value *v)
+{
+    (void) pop(v);
+    if (v->type == STRING) {
+	double d = atof(v->v.string_val);
+	gpfree_string(v);
+	Gcomplex(v, d, 0.);
+	FPRINTF((stderr,"converted string to CMPLX value %g\n",real(v)));
+    }
+    return(v);
+}
+#endif
 
 void
-push(x)
-    struct value *x;
+push(struct value *x)
 {
     if (s_p == STACK_DEPTH - 1)
 	int_error(NO_CARET, "stack overflow");
     stack[++s_p] = *x;
+#ifdef GP_STRING_VARS
+    /* WARNING - This is a memory leak if the string is not later freed */
+    if (x->type == STRING && x->v.string_val)
+	stack[s_p].v.string_val = gp_strdup(x->v.string_val);
+#endif
 }
 
 
 void
-int_check(v)
-    struct value *v;
+int_check(struct value *v)
 {
     if (v->type != INTGR)
 	int_error(NO_CARET, "non-integer passed to boolean operator");
@@ -475,8 +506,7 @@ int_check(v)
 
 /* converts top-of-stack to boolean */
 void
-f_bool(x)
-    union argument *x;
+f_bool(union argument *x)
 {
     (void) x;			/* avoid -Wunused warning */
 
@@ -486,8 +516,7 @@ f_bool(x)
 
 
 void
-f_jump(x)
-    union argument *x;
+f_jump(union argument *x)
 {
     (void) x;			/* avoid -Wunused warning */
     jump_offset = x->j_arg;
@@ -495,8 +524,7 @@ f_jump(x)
 
 
 void
-f_jumpz(x)
-    union argument *x;
+f_jumpz(union argument *x)
 {
     struct value a;
 
@@ -510,8 +538,7 @@ f_jumpz(x)
 
 
 void
-f_jumpnz(x)
-    union argument *x;
+f_jumpnz(union argument *x)
 {
     struct value a;
 
@@ -525,8 +552,7 @@ f_jumpnz(x)
 }
 
 void
-f_jtern(x)
-    union argument *x;
+f_jtern(union argument *x)
 {
     struct value a;
 
@@ -545,7 +571,7 @@ f_jtern(x)
    arguments to be passed to them.
 
    at_ptr is a pointer to the action table which must be executed
-   (evaluated). 
+   (evaluated).
 
    so the iterated line exectues the function indexed by the at_ptr
    and passes the address of the argument which is pointed to by the
@@ -554,10 +580,9 @@ f_jtern(x)
 */
 
 void
-execute_at(at_ptr)
-    struct at_type *at_ptr;
+execute_at(struct at_type *at_ptr)
 {
-    register int instruction_index, operator, count;
+    int instruction_index, operator, count;
     int saved_jump_offset = jump_offset;
 
     count = at_ptr->a_count;
@@ -568,17 +593,15 @@ execute_at(at_ptr)
 	assert(is_jump(operator) || (jump_offset == 1));
 	instruction_index += jump_offset;
     }
-    
+
     jump_offset = saved_jump_offset;
 }
 
 /* 20010724: moved here from parse.c, where it didn't belong */
 void
-evaluate_at(at_ptr, val_ptr)
-    struct at_type *at_ptr;
-    struct value *val_ptr;
+evaluate_at(struct at_type *at_ptr, struct value *val_ptr)
 {
-    double temp;
+    double temp = 0;
 
     undefined = FALSE;
     errno = 0;
@@ -602,6 +625,9 @@ evaluate_at(at_ptr, val_ptr)
 	(void) pop(val_ptr);
 	check_stack();
 	/* At least one machine (ATT 3b1) computes Inf without a SIGFPE */
+#if (GP_STRING_VARS > 1)
+	if (val_ptr->type != STRING)
+#endif
 	temp = real(val_ptr);
 	if (temp > VERYLARGE || temp < -VERYLARGE) {
 	    undefined = TRUE;
@@ -622,4 +648,160 @@ evaluate_at(at_ptr, val_ptr)
 
 }
 
+void
+free_at(struct at_type *at_ptr)
+{
+#ifdef GP_STRING_VARS
+    int i;
+    /* All string constants belonging to this action table have to be
+     * freed before destruction. */
+    if (!at_ptr)
+        return;
+    for(i=0; i<at_ptr->a_count; i++) {
+	struct at_entry *a = &(at_ptr->actions[i]);
+	/* if union a->arg is used as a->arg.v_arg free potential string */
+	if ( a->index == PUSHC || a->index == DOLLARS )
+	    gpfree_string(&(a->arg.v_arg));
+    }
+#endif
+    free(at_ptr);
+}
+
+/* EAM July 2003 - Return pointer to udv with this name; if the key does not
+ * match any existing udv names, create a new one and return a pointer to it.
+ */
+struct udvt_entry *
+add_udv_by_name(char *key)
+{
+    struct udvt_entry **udv_ptr = &first_udv;
+
+    /* check if it's already in the table... */
+
+    while (*udv_ptr) {
+	if (!strcmp(key, (*udv_ptr)->udv_name))
+	    return (*udv_ptr);
+	udv_ptr = &((*udv_ptr)->next_udv);
+    }
+
+    *udv_ptr = (struct udvt_entry *)
+	gp_alloc(sizeof(struct udvt_entry), "value");
+    (*udv_ptr)->next_udv = NULL;
+    (*udv_ptr)->udv_name = gp_strdup(key);
+    (*udv_ptr)->udv_undef = TRUE;
+    return (*udv_ptr);
+}
+
+
+static void fill_gpval_axis __PROTO((AXIS_INDEX axis));
+static void set_gpval_axis_sth_double __PROTO((AXIS_INDEX axis, const char *suffix, double value, int is_int));
+static void fill_gpval_string __PROTO((char *var, const char *value));
+
+static void 
+set_gpval_axis_sth_double(AXIS_INDEX axis, const char *suffix, double value, int is_int)
+{
+    struct udvt_entry *v;
+    char *cc, s[24];
+    sprintf(s, "GPVAL_%s_%s", axis_defaults[axis].name, suffix);
+    for (cc=s; *cc; cc++) *cc = toupper(*cc); /* make the name uppercase */
+    v = add_udv_by_name(s);
+    if (!v) return; /* should not happen */
+    v->udv_undef = FALSE;
+    if (is_int)
+	Ginteger(&v->udv_value, (int)(value+0.5));
+    else
+	Gcomplex(&v->udv_value, value, 0);
+}
+
+static void
+fill_gpval_axis(AXIS_INDEX axis)
+{
+#define A axis_array[axis]
+    double a = AXIS_DE_LOG_VALUE(axis, A.min); /* FIXME GPVAL: This should be replaced by  a = A.real_min  and */
+    double b = AXIS_DE_LOG_VALUE(axis, A.max); /* FIXME GPVAL: b = A.real_max  when true (delogged) min/max range values are implemented in the axis structure */
+    set_gpval_axis_sth_double(axis, "MIN", ((a < b) ? a : b), 0);
+    set_gpval_axis_sth_double(axis, "MAX", ((a < b) ? b : a), 0);
+    set_gpval_axis_sth_double(axis, "REVERSE", (A.range_flags & RANGE_REVERSE), 1);
+    set_gpval_axis_sth_double(axis, "LOG", A.base, 0);
+#undef A
+}
+
+static void
+fill_gpval_string(char *var, const char *stringvalue)
+{
+#ifdef GP_STRING_VARS
+    struct udvt_entry *v = add_udv_by_name(var);
+    if (!v)
+	return;
+    if (v->udv_undef == FALSE && !strcmp(v->udv_value.v.string_val, stringvalue))
+	return;
+    if (v->udv_undef)
+	v->udv_undef = FALSE; 
+    else
+	gpfree_string(&v->udv_value);
+    Gstring(&v->udv_value, gp_strdup(stringvalue));
+#endif
+}
+
+/*
+ * Put all the handling for GPVAL_* variables in this one routine.
+ * We call it from one of several contexts:
+ * 0: following a successful set/unset command
+ * 1: following a successful plot/splot
+ * 2: following an unsuccessful command (int_error)
+ * 3: program entry
+ */
+void
+update_gpval_variables(int context)
+{
+
+    /* These values may change during a plot command due to auto range */
+    if (context == 1) {
+	fill_gpval_axis(FIRST_X_AXIS);
+	fill_gpval_axis(FIRST_Y_AXIS);
+	fill_gpval_axis(SECOND_X_AXIS);
+	fill_gpval_axis(SECOND_Y_AXIS);
+	fill_gpval_axis(FIRST_Z_AXIS);
+	fill_gpval_axis(COLOR_AXIS);
+	fill_gpval_axis(T_AXIS);
+	fill_gpval_axis(U_AXIS);
+	fill_gpval_axis(V_AXIS);
+	return;
+    }
+    
+    /* These are set every time, which is kind of silly because they */
+    /* only change after 'set term' 'set output' ...                 */
+    else {
+	/* FIXME! This preventa a segfault if term==NULL, which can */
+	/* happen if set_terminal() exits via int_error().          */
+	if (!term)
+	    fill_gpval_string("GPVAL_TERM", "unknown");
+	else
+	    fill_gpval_string("GPVAL_TERM", (char *)(term->name));
+	
+	fill_gpval_string("GPVAL_TERMOPTIONS", term_options);
+	fill_gpval_string("GPVAL_OUTPUT", (outstr) ? outstr : "");
+    }
+
+    /* These initializations need only be done once, on program entry */
+    if (context == 3) {
+	struct udvt_entry *v = add_udv_by_name("GPVAL_VERSION");
+	if (v && v->udv_undef == TRUE) {
+	    v->udv_undef = FALSE; 
+	    Gcomplex(&v->udv_value, atof(gnuplot_version), 0);
+	}
+	v = add_udv_by_name("GPVAL_PATCHLEVEL");
+	if (v && v->udv_undef == TRUE) {
+#ifdef GP_STRING_VARS
+	    fill_gpval_string("GPVAL_PATCHLEVEL", gnuplot_patchlevel);
+#else
+	    v->udv_undef = FALSE; 
+	    Ginteger(&v->udv_value, atoi(gnuplot_patchlevel));
+#endif
+	}
+	v = add_udv_by_name("GPVAL_COMPILE_OPTIONS");
+	if (v && v->udv_undef == TRUE)
+	    fill_gpval_string("GPVAL_COMPILE_OPTIONS", compile_options);
+    }
+
+}
 
