@@ -1,5 +1,5 @@
 #ifndef lint
-static char *RCSid() { return RCSid("$Id: plot.c,v 1.70 2004/04/14 15:26:21 broeker Exp $"); }
+static char *RCSid() { return RCSid("$Id: plot.c,v 1.90.2.2 2007/02/09 23:32:08 mikulik Exp $"); }
 #endif
 
 /* GNUPLOT - plot.c */
@@ -67,6 +67,11 @@ static char *RCSid() { return RCSid("$Id: plot.c,v 1.70 2004/04/14 15:26:21 broe
 # include "gpexecute.h"
 #endif
 
+/* on OS/2 this is needed even without USE_MOUSE */
+#if defined(OS2_IPC) && !defined(USE_MOUSE)
+# include "gpexecute.h"
+#endif
+
 /* Used nowhere else */
 #ifdef HAVE_SYS_UTSNAME_H
 # include <sys/utsname.h>
@@ -97,6 +102,8 @@ extern smg$create_key_table();
 # ifndef SIGINT
 #  define SIGINT 2		/* for MSC */
 # endif
+# include "win/wgnuplib.h"
+# include "win/wcommon.h"
 #endif /* _Windows */
 
 /* GNU readline
@@ -104,20 +111,18 @@ extern smg$create_key_table();
  * so I don't put this into a header file. -lh
  */
 #ifdef HAVE_LIBREADLINE
-extern int rl_complete_with_tilde_expansion;
-
-/* enable gnuplot history with readline */
 # ifdef GNUPLOT_HISTORY
 #  include <readline/tilde.h>
-#  ifndef GNUPLOT_HISTORY_FILE
-#   define GNUPLOT_HISTORY_FILE "~/.gnuplot_history"
-#  endif
-#  ifndef HISTORY_SIZE
-/* Can be overriden with the environment variable 'GNUPLOT_HISTORY_SIZE' */
-#   define HISTORY_SIZE 666
-#  endif
-long int gnuplot_history_size = HISTORY_SIZE;
-/* 
+# endif
+extern int rl_complete_with_tilde_expansion;
+#endif 
+
+/* enable gnuplot history with readline */
+#ifdef GNUPLOT_HISTORY
+# ifndef GNUPLOT_HISTORY_FILE
+#  define GNUPLOT_HISTORY_FILE "~/.gnuplot_history"
+# endif
+/*
  * The next variable is a pointer to the value returned from 'tilde_expand()'.
  * This function expands '~' to the user's home directory, or $HOME, with
  * UN*X, AmigaOS, MSDOS.
@@ -128,11 +133,11 @@ static char *expanded_history_filename;
 
 static void wrapper_for_write_history __PROTO((void));
 
-# endif				/* GNUPLOT_HISTORY */
-#endif /* HAVE_LIBREADLINE */
+#endif				/* GNUPLOT_HISTORY */
 
 TBOOLEAN interactive = TRUE;	/* FALSE if stdin not a terminal */
 static TBOOLEAN noinputfiles = TRUE; /* FALSE if there are script files */
+TBOOLEAN persist_cl = FALSE; /* TRUE if -persist is parsed in the command line */
 
 /* user home directory */
 static const char *user_homedir = NULL;
@@ -157,14 +162,13 @@ char HelpFile[MAXPATH];
 /* a longjmp buffer to get back to the command line */
 /* FIXME HBB 20001103: should probably just use GPFAR, rather than
  * check for _Windows */
-#ifdef _Windows 
+#ifdef _Windows
 static JMP_BUF far command_line_env;
 #else
 static JMP_BUF command_line_env;
 #endif
 
 static void load_rcfile __PROTO((void));
-static void get_user_env __PROTO((void));
 static RETSIGTYPE inter __PROTO((int anint));
 static void init_memory __PROTO((void));
 
@@ -174,7 +178,6 @@ static int exit_status = EXIT_SUCCESS;
 # include <process.h>
 static ULONG RexxInterface(PRXSTRING, PUSHORT, PRXSTRING);
 TBOOLEAN CallFromRexx = FALSE;
-void PM_intc_cleanup(void);
 #endif /* OS2 */
 
 #if defined(ATARI) || defined(MTOS)
@@ -186,8 +189,7 @@ extern int aesid;
 #endif
 
 static RETSIGTYPE
-inter(anint)
-    int anint;
+inter(int anint)
 {
     (void) anint;		/* aovid -Wunused warning */
     (void) signal(SIGINT, (sigfunc) inter);
@@ -199,7 +201,7 @@ inter(anint)
 #ifdef OS2
     if (!strcmp(term->name,"pm")) {
         PM_intc_cleanup();
-        /* ?? 
+        /* ??
           putc('\n', stderr);
           LONGJMP(command_line_env, TRUE);
          */
@@ -273,15 +275,14 @@ bail_to_command_line()
 
 #if defined(_Windows) || defined(_Macintosh)
 int
-gnu_main(argc, argv)
+gnu_main(int argc, char **argv)
 #else
 int
-main(argc, argv)
+main(int argc, char **argv)
 #endif
-    int argc;
-    char **argv;
 {
     int i;
+
 #ifdef LINUXVGA
     LINUX_setup();		/* setup VGA before dropping privilege DBT 4/5/99 */
     drop_privilege();
@@ -291,9 +292,11 @@ main(argc, argv)
 #ifdef __linux__
     setuid(getuid());
 #endif
+
 #if defined(MSDOS) && !defined(_Windows) && !defined(__GNUC__)
     PC_setup();
 #endif /* MSDOS !Windows */
+
 /* HBB: Seems this isn't needed any more for DJGPP V2? */
 /* HBB: disable all floating point exceptions, just keep running... */
 #if defined(DJGPP) && (DJGPP!=2)
@@ -375,6 +378,29 @@ main(argc, argv)
 	}
     }
 
+    /* the X11 terminal removes tokens that it recognizes from argv.
+     * We have to parse -persist for the wxWidgets terminal before it happens, and
+     * keep that value for later use */
+    i=0;
+    while (i<argc) {
+	if (!argv[i]) {
+	    ++i;
+	    continue;
+	}
+	if (!strcmp(argv[i], "-persist")) {
+	    FPRINTF((stderr,"'persist' command line option recognized"));
+	    persist_cl = TRUE;
+# ifdef X11
+	    ++i;
+# else
+	    --argc;
+	    ++argv;
+# endif
+	    break;
+	} else
+	    ++i;
+    }
+
 #ifdef X11
     {
 	int n = X11_args(argc, argv);
@@ -428,7 +454,14 @@ main(argc, argv)
 #endif
 
     gpoutfile = stdout;
+
+    /* Initialize pre-loaded user variables */
     (void) Gcomplex(&udv_pi.udv_value, M_PI, 0.0);
+#ifdef HAVE_ISNAN
+    udv_NaN = add_udv_by_name("NaN");
+    (void) Gcomplex(&(udv_NaN->udv_value), atof("NaN"), 0.0);
+    udv_NaN->udv_undef = FALSE;
+#endif
 
     init_memory();
 
@@ -436,6 +469,11 @@ main(argc, argv)
     init_terminal();		/* can set term type if it likes */
     push_terminal(0);		/* remember the default terminal */
 
+    /* reset the terminal when exiting */
+    /* this is done through gp_atexit so that other terminal functions
+     * can be registered to be executed before the terminal is reset. */
+    GP_ATEXIT(term_reset);
+
 #ifdef AMIGA_SC_6_1
     if (IsInteractive(Input()) == DOSTRUE)
 	interactive = TRUE;
@@ -446,16 +484,6 @@ main(argc, argv)
     interactive = TRUE;
 # else
     interactive = isatty(fileno(stdin));
-#ifdef PIPE_IPC
-    /* isatty_state is set here and nowhere else! (used in term/x11.trm) */
-    isatty_state = interactive;
-    if (!isatty_state) {
-	/* stdin is not from a tty --> Turn mouse off.
-	 * can be turned on again, e.g. if the user
-	 * wants to write on a pipe to gnuplot */
-	mouse_setting.on = 0;
-    }
-#endif
 # endif
 #endif /* !AMIGA_SC_6_1 */
 
@@ -499,6 +527,10 @@ main(argc, argv)
 
     if (interactive)
 	show_version(stderr);
+    else
+	show_version(NULL); /* Only load GPVAL_COMPILE_OPTIONS */
+
+    update_gpval_variables(3);  /* update GPVAL_ variables available to user */
 
 #ifdef VMS
     /* initialise screen management routines for command recall */
@@ -524,16 +556,19 @@ main(argc, argv)
 	 * would set them to. Since the axis variables aren't in
 	 * initialized arrays any more, this is now necessary... */
 	reset_command();
-#ifdef PM3D
 	init_color();  /*  Initialization of color  */
-#endif /* PM3D */
 	load_rcfile();
 	init_fit();		/* Initialization of fitting module */
 
 	if (interactive && term != 0) {		/* not unknown */
-#if defined(HAVE_LIBREADLINE) && defined(GNUPLOT_HISTORY)
+#ifdef GNUPLOT_HISTORY
 	    FPRINTF((stderr, "Before read_history\n"));
+#ifdef HAVE_LIBREADLINE
 	    expanded_history_filename = tilde_expand(GNUPLOT_HISTORY_FILE);
+#else
+	    expanded_history_filename = gp_strdup(GNUPLOT_HISTORY_FILE);
+	    gp_expand_tilde(&expanded_history_filename);
+#endif
 	    FPRINTF((stderr, "expanded_history_filename = %s\n", expanded_history_filename));
 	    read_history(expanded_history_filename);
 	    {
@@ -543,7 +578,7 @@ main(argc, argv)
 		    gnuplot_history_size = strtol (temp_env, (char **) NULL, 10);
 	    } /* END: Go local to get environment variable */
 
-	    /* 
+	    /*
 	     * It is safe to ignore the return values of 'atexit()' and
 	     * 'on_exit()'. In the worst case, there is no history of your
 	     * currrent session and you have to type all again in your next
@@ -553,7 +588,7 @@ main(argc, argv)
 	     * want to use them, 'write_history()' is called directly.
 	     */
 	    GP_ATEXIT(wrapper_for_write_history);
-#endif /* HAVE_LIBREADLINE && GNUPLOT_HISTORY */
+#endif /* GNUPLOT_HISTORY */
 
 	    fprintf(stderr, "\nTerminal type set to '%s'\n", term->name);
 	}			/* if (interactive && term != 0) */
@@ -595,7 +630,7 @@ main(argc, argv)
 
     if (argc > 1) {
 #ifdef _Windows
-	int noend = 0;
+	TBOOLEAN noend = persist_cl;
 #endif
 
 	/* load filenames given as arguments */
@@ -605,7 +640,7 @@ main(argc, argv)
 #ifdef _Windows
 	    if (stricmp(*argv, "-noend") == 0 || stricmp(*argv, "/noend") == 0
 	       	|| stricmp(*argv, "-persist") == 0)
-		noend = 1;
+		noend = TRUE;
 	    else
 #endif
 	    if (strcmp(*argv, "-") == 0) {
@@ -616,7 +651,7 @@ main(argc, argv)
 
 		while (!com_line());
 
-/* interactive = FALSE; */ /* should this be here? */
+		interactive = FALSE;
 
 	    } else
 		load_file(loadpath_fopen(*argv, "r"), *argv, FALSE);
@@ -639,8 +674,6 @@ main(argc, argv)
 #endif /* !HAVE_ATEXIT && !HAVE_ON_EXIT */
 #endif /* HAVE_LIBREADLINE && GNUPLOT_HISTORY */
 
-    term_reset();
-
 #ifdef OS2
     RexxDeregisterSubcom("GNUPLOT", NULL);
 #endif
@@ -651,7 +684,7 @@ main(argc, argv)
 #endif
     /* HBB 20040223: Not all compilers like exit() to end main() */
     /* exit(exit_status); */
-    return exit_status;    
+    return exit_status;
 }
 
 
@@ -712,14 +745,15 @@ load_rcfile()
     free(rcfile);
 }
 
-static void
+void
 get_user_env()
 {
     if (user_homedir == NULL) {
 	const char *env_home;
 
-	if ((env_home = getenv(HOME)) 
+	if ((env_home = getenv(HOME))
 #ifdef WIN32
+	    || (env_home = appdata_directory())
 	    || (env_home = getenv("USERPROFILE"))
 #endif
 #ifndef VMS
@@ -758,8 +792,7 @@ get_user_env()
  * we may change that later
  */
 void
-gp_expand_tilde(pathp)
-    char **pathp;
+gp_expand_tilde(char **pathp)
 {
     if (!*pathp)
 	int_error(NO_CARET, "Cannot expand empty path");
@@ -869,7 +902,7 @@ ExecuteMacro(char *argv, int namelength)
         /* Interpreter couldn't be started */
         if (rc == -4)
            /* run was cancelled, but don't give error message */
-            rc = 0;  
+            rc = 0;
     } else if (rc==0) {
         /* all was fine */
     }
@@ -895,14 +928,14 @@ RexxInterface(PRXSTRING rxCmd, PUSHORT pusErr, PRXSTRING rxRc)
 
     memcpy(keepenv, command_line_env, sizeof(JMP_BUF));
     if (!SETJMP(command_line_env, 1)) {
-	/* Set variable input_line.
+	/* Set variable gp_input_line.
 	   Watch out for line length of NOT_ZERO_TERMINATED strings ! */
 	cmdlen = rxCmd->strlength + 1;
 	/* FIXME HBB 20010121: 3rd argument doesn't make sense. Either
-	 * this should be input_line_len, or it shouldn't use
+	 * this should be gp_input_line_len, or it shouldn't use
 	 * safe_strncpy(), here */
-	safe_strncpy(input_line, rxCmd->strptr, cmdlen);
-	input_line[cmdlen] = NUL;
+	safe_strncpy(gp_input_line, rxCmd->strptr, cmdlen);
+	gp_input_line[cmdlen] = NUL;
 	rc = do_line();
 	*pusErr = RXSUBCOM_OK;
 	rxRc->strptr[0] = rc + '0';
@@ -923,7 +956,9 @@ RexxInterface(PRXSTRING rxCmd, PUSHORT pusErr, PRXSTRING rxRc)
 }
 #endif
 
-#if defined(HAVE_LIBREADLINE) && defined(GNUPLOT_HISTORY)
+#ifdef GNUPLOT_HISTORY
+# ifdef HAVE_LIBREADLINE
+
 static void
 wrapper_for_write_history()
 {
@@ -949,4 +984,22 @@ wrapper_for_write_history()
     }
 #endif
 }
-#endif /* HAVE_LIBREADLINE && GNUPLOT_HISTORY */
+
+# else /* HAVE_LIBREADLINE */
+
+/* version for gnuplot's own write_history */
+static void
+wrapper_for_write_history()
+{
+    /* What we really want to do is truncate(expanded_history_filename),
+       but this is only available on BSD compatible systems */
+    remove(expanded_history_filename);
+    if (gnuplot_history_size < 0)
+    	write_history(expanded_history_filename);
+    else
+	write_history_n(gnuplot_history_size, expanded_history_filename, "w");
+}
+
+# endif /* HAVE_LIBREADLINE */
+#endif /* GNUPLOT_HISTORY */
+
