@@ -1,5 +1,5 @@
 #ifndef lint
-static char *RCSid() { return RCSid("$Id: pm3d.c,v 1.44 2004/03/30 11:49:30 broeker Exp $"); }
+static char *RCSid() { return RCSid("$Id: pm3d.c,v 1.66.2.1 2007/02/17 23:53:41 mikulik Exp $"); }
 #endif
 
 /* GNUPLOT - pm3d.c */
@@ -17,27 +17,27 @@ static char *RCSid() { return RCSid("$Id: pm3d.c,v 1.44 2004/03/30 11:49:30 broe
 #ifdef HAVE_CONFIG_H
 # include "config.h"
 #endif
-
-#ifdef PM3D
-
 #include "pm3d.h"
-
 #include "alloc.h"
 #include "axis.h"
+#include "graphics.h"
 #include "graph3d.h"
 #include "hidden3d.h"		/* p_vertex & map3d_xyz() */
+#include "plot2d.h"
 #include "plot3d.h"
 #include "setshow.h"		/* for surface_rot_z */
 #include "term_api.h"		/* for lp_use_properties() */
 #include "command.h"		/* for c_token */
 
+#include <stdlib.h> /* qsort() */
+
 
 /*
   Global options for pm3d algorithm (to be accessed by set / show).
 */
 
 pm3d_struct pm3d = {
-    "",				/* where[6] */
+    "s",			/* where[6] */
     PM3D_FLUSH_BEGIN,		/* flush */
     0,				/* no flushing triangles */
     PM3D_SCANS_AUTOMATIC,	/* scans direction is determined automatically */
@@ -46,14 +46,29 @@ pm3d_struct pm3d = {
 #if PM3D_HAVE_SOLID
     0,				/* solid (off by default, that means `transparent') */
 #endif /* PM3D_HAVE_SOLID */
-    PM3D_IMPLICIT,		/* implicit */
-    PM3D_WHICHCORNER_MEAN	/* color from which corner(s) */
+    PM3D_EXPLICIT,		/* implicit */
+    PM3D_WHICHCORNER_MEAN,	/* color from which corner(s) */
+    1,				/* interpolate along scanline */
+    1				/* interpolate between scanlines */
 };
 
+typedef struct {
+    double gray;
+    double z; /* maximal z value after rotation to graph coordinate system */
+    gpdPoint corners[4];
+    gpiPoint icorners[4]; /* also if EXTENDED_COLOR_SPECS is not defined */
+} quadrangle;
+
+static int allocated_quadrangles = 0;
+static int current_quadrangle = 0;
+static quadrangle* quadrangles = (quadrangle*)0;
+
 /* Internal prototypes for this module */
+static TBOOLEAN plot_has_palette;
+static TBOOLEAN plot_wants_colorbox;
 static double geomean4 __PROTO((double, double, double, double));
 static double median4 __PROTO((double, double, double, double));
-static void pm3d_plot __PROTO((struct surface_points *, char));
+static void pm3d_plot __PROTO((struct surface_points *, int));
 static void pm3d_option_at_error __PROTO((void));
 static void pm3d_rearrange_part __PROTO((struct iso_curve *, const int, struct iso_curve ***, int *));
 static void filled_color_contour_plot  __PROTO((struct surface_points *, int));
@@ -115,6 +130,28 @@ median4 (double x1, double x2, double x3, double x4)
 }
 
 
+/* Minimum of 4 numbers.
+ */
+static double
+minimum4 (double x1, double x2, double x3, double x4)
+{
+    x1 = GPMIN(x1, x2);
+    x3 = GPMIN(x3, x4);
+    return GPMIN(x1, x3);
+}
+
+
+/* Maximum of 4 numbers.
+ */
+static double
+maximum4 (double x1, double x2, double x3, double x4)
+{
+    x1 = GPMAX(x1, x2);
+    x3 = GPMAX(x3, x4);
+    return GPMAX(x1, x3);
+}
+
+
 /*
 * Now the routines which are really just those for pm3d.c
 */
@@ -140,19 +177,20 @@ z2cb(double z)
 
 
 /*
- * Rescale cb value into the interval [0,1].
+ * Rescale cb (color) value into the interval of grays [0,1], taking care
+ * of palette being positive or negative.
  * Note that it is OK for logarithmic cb-axis too.
  */
 double
 cb2gray(double cb)
 {
     if (cb <= CB_AXIS.min)
-	return 0;
+	return (sm_palette.positive == SMPAL_POSITIVE) ? 0 : 1;
     if (cb >= CB_AXIS.max)
-	return 1;
+	return (sm_palette.positive == SMPAL_POSITIVE) ? 1 : 0;
     cb = (cb - CB_AXIS.min)
       / (CB_AXIS.max - CB_AXIS.min);
-    return cb;
+    return (sm_palette.positive == SMPAL_POSITIVE) ? cb : 1-cb;
 }
 
 
@@ -160,11 +198,11 @@ cb2gray(double cb)
  * Rearrange...
  */
 static void
-pm3d_rearrange_part(src, len, dest, invert)
-    struct iso_curve *src;
-    const int len;
-    struct iso_curve ***dest;
-    int *invert;
+pm3d_rearrange_part(
+    struct iso_curve *src,
+    const int len,
+    struct iso_curve ***dest,
+    int *invert)
 {
     struct iso_curve *scanA;
     struct iso_curve *scanB;
@@ -182,7 +220,6 @@ pm3d_rearrange_part(src, len, dest, invert)
     scan_array = *dest = gp_alloc(len * sizeof(scanA), "pm3d scan array");
 
     if (pm3d.direction == PM3D_SCANS_AUTOMATIC) {
-
 	int cnt;
 	int len2 = len;
 	TBOOLEAN exit_outer_loop = 0;
@@ -265,14 +302,17 @@ pm3d_rearrange_part(src, len, dest, invert)
 
 /*
  * Rearrange scan array
- * 
+ *
  * Allocates *first_ptr (and eventually *second_ptr)
  * which must be freed by the caller
  */
 void
-pm3d_rearrange_scan_array(struct surface_points *this_plot,
-			  struct iso_curve ***first_ptr, int *first_n, int *first_invert,
-			  struct iso_curve ***second_ptr, int *second_n, int *second_invert)
+pm3d_rearrange_scan_array(
+    struct surface_points *this_plot,
+    struct iso_curve ***first_ptr,
+    int *first_n, int *first_invert,
+    struct iso_curve ***second_ptr,
+    int *second_n, int *second_invert)
 {
     if (first_ptr) {
 	pm3d_rearrange_part(this_plot->iso_crvs, this_plot->num_iso_read, first_ptr, first_invert);
@@ -297,14 +337,86 @@ pm3d_rearrange_scan_array(struct surface_points *this_plot,
     }
 }
 
+static int compare_quadrangles(const void* v1, const void* v2)
+{
+    const quadrangle* q1 = (const quadrangle*)v1;
+    const quadrangle* q2 = (const quadrangle*)v2;
+
+    if (q1->z > q2->z)
+	return 1;
+    else if (q1->z < q2->z)
+	return -1;
+    else
+	return 0;
+}
+
+void pm3d_depth_queue_clear(void)
+{
+    if (pm3d.direction != PM3D_DEPTH)
+	return;
+
+    if (quadrangles)
+	free(quadrangles);
+    quadrangles = (quadrangle*)0;
+    allocated_quadrangles = 0;
+    current_quadrangle = 0;
+}
+
+void pm3d_depth_queue_flush(void)
+{
+    if (pm3d.direction != PM3D_DEPTH)
+	return;
+
+    if (current_quadrangle > 0 && quadrangles) {
+
+	quadrangle* qp;
+	quadrangle* qe;
+	gpdPoint* gpdPtr;
+	gpiPoint* gpiPtr;
+	vertex out;
+	double z = 0; /* assignment keeps the compiler happy */
+	double w = trans_mat[3][3];
+	int i;
+
+	for (qp = quadrangles, qe = quadrangles + current_quadrangle; qp != qe; qp++) {
+
+	    gpdPtr = qp->corners;
+	    gpiPtr = qp->icorners;
+
+	    for (i = 0; i < 4; i++, gpdPtr++, gpiPtr++) {
+
+		map3d_xyz(gpdPtr->x, gpdPtr->y, gpdPtr->z, &out);
+
+		if (i == 0 || out.z > z)
+		    z = out.z;
+
+		gpiPtr->x = (unsigned int) ((out.x * xscaler / w) + xmiddle);
+		gpiPtr->y = (unsigned int) ((out.y * yscaler / w) + ymiddle);
+	    }
+
+	    qp->z = z; /* maximal z value of all four corners */
+	}
+
+	qsort(quadrangles, current_quadrangle, sizeof (quadrangle), compare_quadrangles);
+
+	for (qp = quadrangles, qe = quadrangles + current_quadrangle; qp != qe; qp++) {
+
+	    set_color(qp->gray);
+	    ifilled_quadrangle(qp->icorners);
+	}
+    }
+
+    pm3d_depth_queue_clear();
+}
 
 /*
  * Now the implementation of the pm3d (s)plotting mode
+ *
+ * Note: the input parameter at_which_z is char, but an old HP cc requires
+ * ANSI C K&R routines with int only.
  */
 static void
-pm3d_plot(this_plot, at_which_z)
-    struct surface_points *this_plot;
-    char at_which_z;
+pm3d_plot(struct surface_points *this_plot, int at_which_z)
 {
     int j, i, i1, ii, ii1, from, curve, scan, up_to, up_to_minus, invert = 0;
     int go_over_pts, max_pts;
@@ -319,6 +431,7 @@ pm3d_plot(this_plot, at_which_z)
 #ifdef EXTENDED_COLOR_SPECS
     gpiPoint icorners[4];
 #endif
+    gpdPoint **bl_point = NULL; /* used for bilinear interpolation */
 
     /* just a shortcut */
     TBOOLEAN color_from_column = this_plot->pm3d_color_from_column;
@@ -347,6 +460,25 @@ pm3d_plot(this_plot, at_which_z)
     curve = 0;
 
     pm3d_rearrange_scan_array(this_plot, &scan_array, &scan_array_n, &invert, (struct iso_curve ***) 0, (int *) 0, (int *) 0);
+
+    if (pm3d.direction == PM3D_DEPTH) {
+
+	for (scan = 0; scan < this_plot->num_iso_read - 1; scan++) {
+
+	    scanA = scan_array[scan];
+	    scanB = scan_array[scan + 1];
+
+	    are_ftriangles = pm3d.ftriangles && (scanA->p_count != scanB->p_count);
+	    if (!are_ftriangles)
+		allocated_quadrangles += GPMIN(scanA->p_count, scanB->p_count) - 1;
+	    else {
+		allocated_quadrangles += GPMAX(scanA->p_count, scanB->p_count) - 1;
+	    }
+	}
+
+	quadrangles = (quadrangle*)gp_realloc(quadrangles, allocated_quadrangles * sizeof (quadrangle), "pm3d_plot->quadrangles");
+	/* DEBUG: fprintf(stderr, "allocated_quadrangles = %d\n", allocated_quadrangles); */
+    }
     /* pm3d_rearrange_scan_array(this_plot, (struct iso_curve***)0, (int*)0, &scan_array, &invert); */
 
 #if 0
@@ -371,6 +503,15 @@ pm3d_plot(this_plot, at_which_z)
     printf("\n");
 #endif
 
+    /*
+     * if bilinear interpolation is enabled, allocate memory for the
+     * interpolated points here
+     */
+    if (pm3d.interp_i > 1 || pm3d.interp_j > 1) {
+	bl_point = (gpdPoint **)gp_alloc(sizeof(gpdPoint*) * (pm3d.interp_i+1), "bl-interp along scan");
+	for (i1 = 0; i1 <= pm3d.interp_i; i1++)
+	    bl_point[i1] = (gpdPoint *)gp_alloc(sizeof(gpdPoint) * (pm3d.interp_j+1), "bl-interp between scan");
+    }
 
     /*
      * this loop does the pm3d draw of joining two curves
@@ -405,15 +546,15 @@ pm3d_plot(this_plot, at_which_z)
 	else {
 	    max_pts = GPMAX(scanA->p_count, scanB->p_count);
 	    go_over_pts = max_pts - 1;
-	    /* the j-subrange of quadrangles; in the remaing of the interval 
+	    /* the j-subrange of quadrangles; in the remaing of the interval
 	     * [0..up_to] the flushing triangles are to be drawn */
 	    ftriangles_low_pt = from;
 	    ftriangles_high_pt = from + up_to_minus;
 	}
-	/* Go over 
+	/* Go over
 	 *   - the minimal number of points from both scans, if only quadrangles.
 	 *   - the maximal number of points from both scans if flush triangles
-	 *     (the missing points in the scan of lower nb of points will be 
+	 *     (the missing points in the scan of lower nb of points will be
 	 *     duplicated from the begin/end points).
 	 *
 	 * Notice: if it would be once necessary to go over points in `backward'
@@ -486,6 +627,8 @@ pm3d_plot(this_plot, at_which_z)
 		    case PM3D_WHICHCORNER_MEAN: avgC = (cb1 + cb2 + cb3 + cb4) * 0.25; break;
 		    case PM3D_WHICHCORNER_GEOMEAN: avgC = geomean4(cb1, cb2, cb3, cb4); break;
 		    case PM3D_WHICHCORNER_MEDIAN: avgC = median4(cb1, cb2, cb3, cb4); break;
+		    case PM3D_WHICHCORNER_MIN: avgC = minimum4(cb1, cb2, cb3, cb4); break;
+		    case PM3D_WHICHCORNER_MAX: avgC = maximum4(cb1, cb2, cb3, cb4); break;
 		    case PM3D_WHICHCORNER_C1: avgC = cb1; break;
 		    case PM3D_WHICHCORNER_C2: avgC = cb2; break;
 		    case PM3D_WHICHCORNER_C3: avgC = cb3; break;
@@ -497,7 +640,7 @@ pm3d_plot(this_plot, at_which_z)
 #if 0
 		/* print the quadrangle with the given color */
 		printf("averageColor %g\tgray=%g\tM %g %g L %g %g L %g %g L %g %g\n",
-		       avgColor,
+		       avgC,
 		       gray,
 		       pointsA[i].x, pointsA[i].y, pointsB[ii].x, pointsB[ii].y,
 		       pointsB[ii1].x, pointsB[ii1].y, pointsA[i1].x, pointsA[i1].y);
@@ -516,7 +659,7 @@ pm3d_plot(this_plot, at_which_z)
 	    corners[3].x = pointsA[i1].x;
 	    corners[3].y = pointsA[i1].y;
 
-	    if (at_which_z == PM3D_AT_SURFACE) {
+	    if ( pm3d.interp_i > 1 || pm3d.interp_j > 1 || at_which_z == PM3D_AT_SURFACE) {
 		/* always supply the z value if
 		 * EXTENDED_COLOR_SPECS is defined
 		 */
@@ -540,21 +683,129 @@ pm3d_plot(this_plot, at_which_z)
 		    icorners[3].z = pointsA[i1].z;
 		}
 		for (i = 0; i < 4; i++) {
-		    icorners[i].spec.gray = 
+		    icorners[i].spec.gray =
 			cb2gray( color_from_column ? icorners[i].z : z2cb(icorners[i].z) );
 		}
 	    }
+	    if (pm3d.direction == PM3D_DEPTH) {
+
+		/* copy quadrangle */
+		quadrangle* qp = quadrangles + current_quadrangle;
+		memcpy(qp->corners, corners, 4 * sizeof (gpdPoint));
+		qp->gray = gray;
+		for (i = 0; i < 4; i++) {
+		    qp->icorners[i].z = icorners[i].z;
+		    qp->icorners[i].spec.gray = icorners[i].spec.gray;
+		}
+		current_quadrangle++;
+
+	    } else
 	    filled_quadrangle(corners, icorners);
 #else
-	    /* filled_polygon( 4, corners ); */
-	    filled_quadrangle(corners);
+	    if (pm3d.direction == PM3D_DEPTH) {
+
+		/* copy quadrangle */
+		quadrangle* qp = quadrangles + current_quadrangle;
+		memcpy(qp->corners, corners, 4 * sizeof (gpdPoint));
+		qp->gray = gray;
+		current_quadrangle++;
+
+	    } else
+	    if (pm3d.interp_i > 1 || pm3d.interp_j > 1) {
+		/* Interpolation is enabled.
+		 * interp_i is the # of points along scan lines
+		 * interp_j is the # of points between scan lines
+		 * Algorithm is to first sample i points along the scan lines
+		 * defined by corners[3],corners[0] and corners[2],corners[1]. */
+		int j1;
+		for (i1 = 0; i1 <= pm3d.interp_i; i1++) {
+		    bl_point[i1][0].x = 
+			((corners[3].x - corners[0].x) / pm3d.interp_i) * i1 + corners[0].x;
+		    bl_point[i1][pm3d.interp_j].x = 
+			((corners[2].x - corners[1].x) / pm3d.interp_i) * i1 + corners[1].x;
+		    bl_point[i1][0].y =
+			((corners[3].y - corners[0].y) / pm3d.interp_i) * i1 + corners[0].y;
+		    bl_point[i1][pm3d.interp_j].y =
+			((corners[2].y - corners[1].y) / pm3d.interp_i) * i1 + corners[1].y;
+		    bl_point[i1][0].z =
+			((corners[3].z - corners[0].z) / pm3d.interp_i) * i1 + corners[0].z;
+		    bl_point[i1][pm3d.interp_j].z =
+			((corners[2].z - corners[1].z) / pm3d.interp_i) * i1 + corners[1].z;
+		    /* Next we sample j points between each of the new points
+		     * created in the previous step (this samples between
+		     * scan lines) in the same manner. */
+		    for (j1 = 1; j1 < pm3d.interp_j; j1++) {
+			bl_point[i1][j1].x =
+			    ((bl_point[i1][pm3d.interp_j].x - bl_point[i1][0].x) / pm3d.interp_j) *
+				j1 + bl_point[i1][0].x;
+			bl_point[i1][j1].y =
+			    ((bl_point[i1][pm3d.interp_j].y - bl_point[i1][0].y) / pm3d.interp_j) *
+				j1 + bl_point[i1][0].y;
+			bl_point[i1][j1].z =
+			    ((bl_point[i1][pm3d.interp_j].z - bl_point[i1][0].z) / pm3d.interp_j) *
+				j1 + bl_point[i1][0].z;
+		    }
+		}
+		/* Once all points are created, move them into an appropriate
+		 * structure and call set_color on each to retrieve the
+		 * correct color mapping for this new sub-sampled quadrangle. */
+		for (i1 = 0; i1 < pm3d.interp_i; i1++) {
+		    for (j1 = 0; j1 < pm3d.interp_j; j1++) {
+			corners[0].x = bl_point[i1][j1].x;
+			corners[0].y = bl_point[i1][j1].y;
+			corners[0].z = bl_point[i1][j1].z;
+			corners[1].x = bl_point[i1+1][j1].x;
+			corners[1].y = bl_point[i1+1][j1].y;
+			corners[1].z = bl_point[i1+1][j1].z;
+			corners[2].x = bl_point[i1+1][j1+1].x;
+			corners[2].y = bl_point[i1+1][j1+1].y;
+			corners[2].z = bl_point[i1+1][j1+1].z;
+			corners[3].x = bl_point[i1][j1+1].x;
+			corners[3].y = bl_point[i1][j1+1].y;
+			corners[3].z = bl_point[i1][j1+1].z;
+#if 0
+			printf("(%g,%g),(%g,%g),(%g,%g),(%g,%g)\n",
+			    corners[0].x, corners[0].y,
+			    corners[1].x, corners[1].y,
+			    corners[2].x, corners[2].y,
+			    corners[3].x, corners[3].y);
+#endif
+			cb1 = z2cb(corners[0].z);
+		    	cb2 = z2cb(corners[1].z);
+	    		cb3 = z2cb(corners[2].z);
+    			cb4 = z2cb(corners[3].z);
+			switch (pm3d.which_corner_color) {
+			    case PM3D_WHICHCORNER_MEAN: avgC = (cb1 + cb2 + cb3 + cb4) * 0.25; break;
+			    case PM3D_WHICHCORNER_GEOMEAN: avgC = geomean4(cb1, cb2, cb3, cb4); break;
+			    case PM3D_WHICHCORNER_MEDIAN: avgC = median4(cb1, cb2, cb3, cb4); break;
+			    case PM3D_WHICHCORNER_MIN: avgC = minimum4(cb1, cb2, cb3, cb4); break;
+			    case PM3D_WHICHCORNER_MAX: avgC = maximum4(cb1, cb2, cb3, cb4); break;
+			    case PM3D_WHICHCORNER_C1: avgC = cb1; break;
+			    case PM3D_WHICHCORNER_C2: avgC = cb2; break;
+			    case PM3D_WHICHCORNER_C3: avgC = cb3; break;
+			    case PM3D_WHICHCORNER_C4: avgC = cb4; break;
+			    default: int_error(NO_CARET, "cannot be here"); avgC = 0;
+			}
+			/* transform z value to gray, i.e. to interval [0,1] */
+			gray = cb2gray(avgC);
+			set_color(gray);
+			filled_quadrangle(corners);
+		    }
+		}
+	    } else {
+		filled_quadrangle(corners);
+	    } /* interpolate between points */
 #endif
 	} /* loop quadrangles over points of two subsequent scans */
     } /* loop over scans */
 
+    if (bl_point) {
+	for (i1 = 0; i1 <= pm3d.interp_i; i1++)
+	    free(bl_point[i1]);
+	free(bl_point);
+    }
     /* free memory allocated by scan_array */
     free(scan_array);
-
 }				/* end of pm3d splotting mode */
 
 
@@ -562,9 +813,7 @@ pm3d_plot(this_plot, at_which_z)
  *  Now the implementation of the filled color contour plot
 */
 static void
-filled_color_contour_plot(this_plot, contours_where)
-    struct surface_points *this_plot;
-    int contours_where;
+filled_color_contour_plot(struct surface_points *this_plot, int contours_where)
 {
     double gray;
     struct gnuplot_contours *cntr;
@@ -613,9 +862,9 @@ filled_color_contour_plot(this_plot, contours_where)
  * unset pm3d for the reset command
  */
 void
-pm3d_reset(void)
+pm3d_reset()
 {
-    pm3d.where[0] = 0;
+    strcpy(pm3d.where, "s");
     pm3d.flush = PM3D_FLUSH_BEGIN;
     pm3d.ftriangles = 0;
     pm3d.direction = PM3D_SCANS_AUTOMATIC;
@@ -624,12 +873,14 @@ pm3d_reset(void)
 #if PM3D_HAVE_SOLID
     pm3d.solid = 0;
 #endif /* PM3D_HAVE_SOLID */
-    pm3d.implicit = PM3D_IMPLICIT;
+    pm3d.implicit = PM3D_EXPLICIT;
     pm3d.which_corner_color = PM3D_WHICHCORNER_MEAN;
+    pm3d.interp_i = 1;
+    pm3d.interp_j = 1;
 }
 
 
-/* 
+/*
  * Draw (one) PM3D color surface.
  */
 void
@@ -645,8 +896,8 @@ pm3d_draw_one(struct surface_points *plot)
     }
 
     /* for pm3dCompress.awk */
-    if (postscript_gpoutfile)
-	fprintf(gpoutfile, "%%pm3d_map_begin\n");
+    if (gppsfile)
+	fputs("%pm3d_map_begin\n", gppsfile);
 
     for (; where[i]; i++) {
 	pm3d_plot(plot, where[i]);
@@ -654,7 +905,7 @@ pm3d_draw_one(struct surface_points *plot)
 
     if (strchr(where, 'C') != NULL) {
 	/* !!!!! FILLED COLOR CONTOURS, *UNDOCUMENTED*
-	   !!!!! LATER CHANGE TO STH LIKE 
+	   !!!!! LATER CHANGE TO STH LIKE
 	   !!!!!   (if_filled_contours_requested)
 	   !!!!!      ...
 	   Currently filled color contours do not work because gnuplot generates
@@ -667,8 +918,8 @@ pm3d_draw_one(struct surface_points *plot)
     }
 
     /* for pm3dCompress.awk */
-    if (postscript_gpoutfile)
-	fprintf(postscript_gpoutfile, "%%pm3d_map_end\n");
+    if (gppsfile)
+	fputs("%pm3d_map_end\n", gppsfile);
 }
 
 
@@ -676,9 +927,11 @@ pm3d_draw_one(struct surface_points *plot)
  */
 
 static void
-pm3d_option_at_error(void)
+pm3d_option_at_error()
 {
-    int_error(c_token,"parameter to `pm3d at` requires combination of up to 6 characters b,s,t\n\t(drawing at bottom, surface, top)");
+    int_error(c_token, "\
+parameter to `pm3d at` requires combination of up to 6 characters b,s,t\n\
+\t(drawing at bottom, surface, top)");
 }
 
 
@@ -692,11 +945,13 @@ int
 get_pm3d_at_option(char *pm3d_where)
 {
     char* c;
+
     if (END_OF_COMMAND || token[c_token].length >= sizeof(pm3d.where)) {
 	pm3d_option_at_error();
 	return 1;
     }
-    memcpy(pm3d_where, input_line + token[c_token].start_index, token[c_token].length);
+    memcpy(pm3d_where, gp_input_line + token[c_token].start_index,
+	   token[c_token].length);
     pm3d_where[ token[c_token].length ] = 0;
     /* verify the parameter */
     for (c = pm3d_where; *c; c++) {
@@ -710,5 +965,107 @@ get_pm3d_at_option(char *pm3d_where)
     return 0;
 }
 
+/* Set flag plot_has_palette to TRUE if there is any element on the graph
+ * which requires palette of continuous colors.
+ */
+void
+set_plot_with_palette(int plot_num, int plot_mode)
+{
+    struct surface_points *this_3dplot = first_3dplot;
+    struct curve_points *this_2dplot = first_plot;
+    int surface = 0;
+    struct text_label *this_label = first_label;
+    TBOOLEAN want_palette_but_not_colorbox = FALSE;
 
+    plot_has_palette = TRUE;
+    plot_wants_colorbox = TRUE;
+    /* Is pm3d switched on globally? */
+    if (pm3d.implicit == PM3D_IMPLICIT)
+	return;
+
+    /* Check 2D plots */
+    if (plot_mode == MODE_PLOT) {
+	while (this_2dplot) {
+#ifdef WITH_IMAGE
+	    if (this_2dplot->plot_style == IMAGE)
+		return;
 #endif
+	    if (this_2dplot->lp_properties.use_palette) {
+		if (this_2dplot->lp_properties.pm3d_color.type <= TC_RGB)
+		    want_palette_but_not_colorbox = TRUE;
+		    /* don't return yet -- decide later whether showing color box is desirable */
+		else
+		    return;
+	    }
+#ifdef EAM_DATASTRINGS
+	    if (this_2dplot->labels &&
+		this_2dplot->labels->textcolor.type >= TC_CB)
+		return;
+#endif
+	    this_2dplot = this_2dplot->next;
+	}
+    }
+
+    /* Check 3D plots */
+    if (plot_mode == MODE_SPLOT) {
+	/* Any surface 'with pm3d', 'with image' or 'with line|dot palette'? */
+	while (surface++ < plot_num) {
+	    if (this_3dplot->plot_style == PM3DSURFACE)
+		return;
+#ifdef WITH_IMAGE
+	    if (this_3dplot->plot_style == IMAGE)
+		return;
+#endif
+	    if (this_3dplot->lp_properties.use_palette) {
+	        int type = this_3dplot->lp_properties.pm3d_color.type;
+		if (type == TC_LT || type == TC_LINESTYLE || type == TC_RGB)
+		    want_palette_but_not_colorbox = TRUE;
+		    /* don't return yet -- decide later whether showing color box is desirable */
+		else
+		    /* TC_DEFAULT: splot x with line|lp|dot palette */
+		    return;
+	    }
+#ifdef EAM_DATASTRINGS
+	    if (this_3dplot->labels &&
+		this_3dplot->labels->textcolor.type >= TC_CB)
+		return;
+#endif
+	    this_3dplot = this_3dplot->next_sp;
+	}
+    }
+
+    /* Any label with 'textcolor palette'? */
+#define TC_USES_PALETTE(tctype) (tctype==TC_Z) || (tctype==TC_CB) || (tctype==TC_FRAC)
+    for (; this_label != NULL; this_label = this_label->next) {
+	if (TC_USES_PALETTE(this_label->textcolor.type))
+	    return;
+    }
+    /* Any of title, xlabel, ylabel, zlabel, ... with 'textcolor palette'? */
+    if (TC_USES_PALETTE(title.textcolor.type)) return;
+    if (TC_USES_PALETTE(axis_array[FIRST_X_AXIS].label.textcolor.type)) return;
+    if (TC_USES_PALETTE(axis_array[FIRST_Y_AXIS].label.textcolor.type)) return;
+    if (TC_USES_PALETTE(axis_array[SECOND_X_AXIS].label.textcolor.type)) return;
+    if (TC_USES_PALETTE(axis_array[SECOND_Y_AXIS].label.textcolor.type)) return;
+    if (plot_mode == MODE_SPLOT)
+	if (TC_USES_PALETTE(axis_array[FIRST_Z_AXIS].label.textcolor.type)) return;
+    if (TC_USES_PALETTE(axis_array[COLOR_AXIS].label.textcolor.type)) return;
+#undef TC_USES_PALETTE
+
+    /* Palette with continuous colors is not used. */
+    if (want_palette_but_not_colorbox == FALSE)
+	plot_has_palette = FALSE; /* otherwise it stays TRUE */
+    plot_wants_colorbox = FALSE;
+}
+
+TBOOLEAN
+is_plot_with_palette()
+{
+    return plot_has_palette;
+}
+
+TBOOLEAN
+is_plot_with_colorbox()
+{
+    return plot_wants_colorbox;
+}
+
