@@ -1,5 +1,5 @@
 #ifndef lint
-static char *RCSid() { return RCSid("$Id: specfun.c,v 1.27 2004/04/13 17:24:01 broeker Exp $"); }
+static char *RCSid() { return RCSid("$Id: specfun.c,v 1.34 2005/04/22 21:40:37 broeker Exp $"); }
 #endif
 
 /* GNUPLOT - specfun.c */
@@ -58,8 +58,13 @@ static char *RCSid() { return RCSid("$Id: specfun.c,v 1.27 2004/04/13 17:24:01 b
 # define MACHEPS 1.0E-08
 #endif
 
+#ifndef E_MINEXP
 /* AS239 value, e^-88 = 2^-127 */
-#define MINEXP  (-88.0)
+#define E_MINEXP  (-88.0)
+#endif
+#ifndef E_MAXEXP
+#define E_MAXEXP (-E_MINEXP)
+#endif
 
 #ifdef FLT_MAX
 # define OFLOW   FLT_MAX		/* 1.0E+37 */
@@ -130,13 +135,6 @@ static double erfc __PROTO((double a));
  */
 #define UNK 1
 
-/* If you define UNK, then be sure to set BIGENDIAN properly. */
-#ifdef FLOAT_WORDS_BIGENDIAN
-#define BIGENDIAN 1
-#else
-#define BIGENDIAN 0
-#endif
-
 /* Define to support tiny denormal numbers, else undefine. */
 #define DENORMAL 1
 
@@ -156,7 +154,7 @@ Copyright 1984, 1987 by Stephen L. Moshier
 Direct inquiries to 30 Frost Street, Cambridge, MA 02140
 */
 
-int             merror = 0;
+static int merror = 0;
 
 /* Notice: the order of appearance of the following messages cannot be bound
  * to error codes defined in mconf.h or math.h or similar, as these files are
@@ -170,9 +168,7 @@ int             merror = 0;
 #define MTHERR_PLPREC	 6
 
 static int
-mtherr(name, code)
-    char *name;
-    int code;
+mtherr(char *name, int code)
 {
     static const char *ermsg[7] = {
 	"unknown",                  /* error code 0 */
@@ -251,16 +247,12 @@ Cephes Math Library Release 2.1:  December, 1988
 Copyright 1984, 1987, 1988 by Stephen L. Moshier
 Direct inquiries to 30 Frost Street, Cambridge, MA 02140
 */
-
 static double
-polevl(x, coef, N)
-    double x;
-    const double coef[];
-    int N;
+polevl(double x, const double coef[], int N)
 {
     double          ans;
     int             i;
-    const double         *p;
+    const double    *p;
 
     p = coef;
     ans = *p++;
@@ -277,16 +269,12 @@ polevl(x, coef, N)
  * Evaluate polynomial when coefficient of x  is 1.0.
  * Otherwise same as polevl.
  */
-
 static double
-p1evl(x, coef, N)
-    double x;
-    const double coef[];
-    int N;
+p1evl(double x, const double coef[], int N)
 {
-    double          ans;
-    const double         *p;
-    int             i;
+    double		ans;
+    const double	*p;
+    int		 	i;
 
     p = coef;
     ans = x + *p++;
@@ -306,28 +294,27 @@ p1evl(x, coef, N)
 int             sgngam;
 
 static int
-ISNAN(x)
-    double x;
+ISNAN(double x)
 {
     volatile double a = x;
+
     if (a != a)
         return 1;
     return 0;
 }
 
 static int
-ISFINITE(x)
-    double x;
+ISFINITE(double x)
 {
     volatile double a = x;
+
     if (a < DBL_MAX)
         return 1;
     return 0;
 }
 
 double
-lngamma(x)
-    double x;
+lngamma(double x)
 {
     /* A[]: Stirling's formula expansion of log gamma
      * B[], C[]: log gamma function between 2 and 3
@@ -562,68 +549,31 @@ lngamma(x)
 #endif /* !GAMMA */
 
 void
-f_erf(arg)
-    union argument *arg;
+f_erf(union argument *arg)
 {
     struct value a;
     double x;
 
     (void) arg;				/* avoid -Wunused warning */
     x = real(pop(&a));
-
-#ifdef HAVE_ERF
     x = erf(x);
-#else
-    /* HBB 20031006 FIXME: why is this not using the erf()
-     * implementation found further down in this source file? */
-    {
-	int fsign = (x >= 0);
-
-	x = igamma(0.5, (x) * (x));
-	if (x == -1.0) {
-	    undefined = TRUE;
-	    x = 0.0;
-	} else {
-	    if (! fsign)
-		x = -x;
-	}
-    }
-#endif
     push(Gcomplex(&a, x, 0.0));
 }
 
 void
-f_erfc(arg)
-    union argument *arg;
+f_erfc(union argument *arg)
 {
     struct value a;
     double x;
 
     (void) arg;				/* avoid -Wunused warning */
     x = real(pop(&a));
-#ifdef HAVE_ERFC
     x = erfc(x);
-#else
-    /* HBB 20031006 FIXME: why is this not using the erfc()
-     * implementation found further down in this source file? */
-    {
-	int fsign = (x >= 0);
-
-	x = igamma(0.5, (x) * (x));
-	if (x == -1.0) {
-	    undefined = TRUE;
-	    x = 0.0;
-	} else { 
-	    x = fsign ? (1.0 - x) : (1.0 + x);
-	}
-    }
-#endif
     push(Gcomplex(&a, x, 0.0));
 }
 
 void
-f_ibeta(arg)
-    union argument *arg;
+f_ibeta(union argument *arg)
 {
     struct value a;
     double x;
@@ -643,8 +593,7 @@ f_ibeta(arg)
 	push(Gcomplex(&a, x, 0.0));
 }
 
-void f_igamma(arg)
-    union argument *arg;
+void f_igamma(union argument *arg)
 {
     struct value a;
     double x;
@@ -662,23 +611,21 @@ void f_igamma(arg)
 	push(Gcomplex(&a, x, 0.0));
 }
 
-void f_gamma(arg)
-    union argument *arg;
+void f_gamma(union argument *arg)
 {
-    register double y;
+    double y;
     struct value a;
 
     (void) arg;				/* avoid -Wunused warning */
     y = GAMMA(real(pop(&a)));
-    if (y > -MINEXP) {
+    if (y > E_MAXEXP) {
 	undefined = TRUE;
 	push(Ginteger(&a, 0));
     } else
 	push(Gcomplex(&a, signgam * gp_exp(y), 0.0));
 }
 
-void f_lgamma(arg)
-    union argument *arg;
+void f_lgamma(union argument *arg)
 {
     struct value a;
 
@@ -688,8 +635,7 @@ void f_lgamma(arg)
 
 #ifndef BADRAND
 
-void f_rand(arg)
-    union argument *arg;
+void f_rand(union argument *arg)
 {
     struct value a;
 
@@ -700,8 +646,7 @@ void f_rand(arg)
 #else /* BADRAND */
 
 /* Use only to observe the effect of a "bad" random number generator. */
-void f_rand(arg)
-    union argument *arg;
+void f_rand(union argument *arg)
 {
     struct value a;
 
@@ -754,8 +699,7 @@ void f_rand(arg)
  */
 
 static double
-ibeta(a, b, x)
-    double a, b, x;
+ibeta(double a, double b, double x)
 {
     /* Test for admissibility of arguments */
     if (a <= 0.0 || b <= 0.0)
@@ -772,8 +716,7 @@ ibeta(a, b, x)
 }
 
 static double
-confrac(a, b, x)
-    double a, b, x;
+confrac(double a, double b, double x)
 {
     double Alo = 0.0;
     double Ahi;
@@ -853,8 +796,7 @@ confrac(a, b, x)
  */
 
 static double
-igamma(a, x)
-    double a, x;
+igamma(double a, double x)
 {
     double arg;
     double aa;
@@ -961,8 +903,7 @@ igamma(a, x)
      Software, 17:98-111 (1991)
 ***********************************************************************/
 static double
-ranf(init)
-    struct value *init;
+ranf(struct value *init)
 {
     long k, z;
     static int firsttime = 1;
@@ -993,7 +934,7 @@ ranf(init)
 	    seed2 = seed1;
     }
     FPRINTF((stderr,"ranf: seed = %lo %lo        %ld %ld\n", seed1,seed2));
-    
+
     /* Generate pseudo random integers */
     k = seed1 / 53668L;
     seed1 = Xa1 * (seed1 - k * 53668L) - k * 12211;
@@ -1020,44 +961,26 @@ ranf(init)
    ---------------------------------------------------------------- */
 
 void
-f_normal(arg)
-    union argument *arg;
+f_normal(union argument *arg)
 {				/* Normal or Gaussian Probability Function */
     struct value a;
     double x;
 
-    /* ref. Abramowitz and Stegun 1964, "Handbook of Mathematical 
+    /* ref. Abramowitz and Stegun 1964, "Handbook of Mathematical
        Functions", Applied Mathematics Series, vol 55,
-       Chapter 26, page 934, Eqn. 26.2.29 and Jos van der Woude 
+       Chapter 26, page 934, Eqn. 26.2.29 and Jos van der Woude
        code found above */
 
     (void) arg;				/* avoid -Wunused warning */
     x = real(pop(&a));
 
     x = 0.5 * SQRT_TWO * x;
-#ifdef HAVE_ERF
     x = 0.5 * (1.0 + erf(x));
-#else
-    {
-	int fsign;
-	fsign = x >= 0 ? 1 : 0;
-	x = igamma(0.5, (x) * (x));
-	if (x == 1.0) {
-	    undefined = TRUE;
-	    x = 0.0;
-	} else { 
-	    if (fsign == 0)
-		x = -(x);
-	    x = 0.5 * (1.0 + x);
-	}
-    }
-#endif
     push(Gcomplex(&a, x, 0.0));
 }
 
 void
-f_inverse_normal(arg)
-    union argument *arg;
+f_inverse_normal(union argument *arg)
 {				/* Inverse normal distribution function */
     struct value a;
     double x;
@@ -1075,8 +998,7 @@ f_inverse_normal(arg)
 
 
 void
-f_inverse_erf(arg)
-    union argument *arg;
+f_inverse_erf(union argument *arg)
 {				/* Inverse error function */
     struct value a;
     double x;
@@ -1167,8 +1089,7 @@ static unsigned short s2p[] = {
 #endif
 
 static double
-inverse_normal_func(y0)
-    double y0;
+inverse_normal_func(double y0)
 {
     /* approximation for 0 <= |y - 0.5| <= 3/8 */
 #ifdef UNK
@@ -1547,8 +1468,7 @@ Copyright 1984, 1987, 1988, 1992, 2000 by Stephen L. Moshier
  */
 
 static double
-erfc(a)
-    double a;
+erfc(double a)
 {
 #ifdef UNK
     static const double   P[] = {
@@ -1801,12 +1721,11 @@ erfc(a)
  *
  */
 
-double
-erf(x)
-    double x;
+static double
+erf(double x)
 {
 
-#ifdef UNK
+# ifdef UNK
     static const double   T[] = {
 	9.60497373987051638749E0,
 	9.00260197203842689217E1,
@@ -1822,9 +1741,9 @@ erf(x)
 	2.26290000613890934246E4,
 	4.92673942608635921086E4
     };
-#endif
+# endif
 
-#ifdef DEC
+# ifdef DEC
     static const unsigned short T[] = {
 	0041031, 0126770, 0170672, 0166101,
 	0041664, 0006522, 0072360, 0031770,
@@ -1840,9 +1759,9 @@ erf(x)
 	0043660, 0145000, 0004013, 0160114,
 	0044100, 0071544, 0167107, 0125471
     };
-#endif
+# endif
 
-#ifdef IBMPC
+# ifdef IBMPC
     static const unsigned short T[] = {
 	0x5d88, 0x1e37, 0x35bf, 0x4023,
 	0x067f, 0x4e9e, 0x81aa, 0x4056,
@@ -1858,9 +1777,9 @@ erf(x)
 	0x7c0a, 0x0101, 0x1940, 0x40d6,
 	0xf567, 0x9dc8, 0x0e6c, 0x40e8
     };
-#endif
+# endif
 
-#ifdef MIEEE
+# ifdef MIEEE
     static const unsigned short T[] = {
 	0x4023, 0x35bf, 0x1e37, 0x5d88,
 	0x4056, 0x81aa, 0x4e9e, 0x067f,
@@ -1875,7 +1794,7 @@ erf(x)
 	0x40d6, 0x1940, 0x0101, 0x7c0a,
 	0x40e8, 0x0e6c, 0x9dc8, 0xf567
     };
-#endif
+# endif
 
     double y, z;
 
@@ -1884,64 +1803,19 @@ erf(x)
     z = x * x;
     y = x * polevl(z, T, 4) / p1evl(z, U, 5);
     return (y);
-
 }
-#endif
+#endif /* !HAVE_ERF */
 
-/*                                                      ndtr.c
- *
- *      Normal distribution function
- *
- *
- *
- * SYNOPSIS:
- *
- * double x, y, ndtr();
- *
- * y = ndtr( x );
- *
- *
- *
- * DESCRIPTION:
- *
- * Returns the area under the Gaussian probability density
- * function, integrated from minus infinity to x:
- *
- *                            x
- *                             -
- *                   1        | |          2
- *    ndtr(x)  = ---------    |    exp( - t /2 ) dt
- *               sqrt(2pi)  | |
- *                           -
- *                          -inf.
- *
- *             =  ( 1 + erf(z) ) / 2
- *             =  erfc(z) / 2
- *
- * where z = x/sqrt(2). Computation is via the functions
- * erf and erfc.
- *
- *
- * ACCURACY:
- *
- *                      Relative error:
- * arithmetic   domain     # trials      peak         rms
- *    DEC      -13,0         8000       2.1e-15     4.8e-16
- *    IEEE     -13,0        30000       3.4e-14     6.7e-15
- *
- *
- * ERROR MESSAGES:
- *
- *   message         condition         value returned
- * erfc underflow    x > 37.519379347       0.0
- *
+/* ----------------------------------------------------------------
+   Following function for the inverse error function is taken from
+   NIST on 16. May 2002.
+   Use Newton-Raphson correction also for range -1 to -y0 and
+   add 3rd cycle to improve convergence -  E A Merritt 21.10.2003
+   ----------------------------------------------------------------
  */
 
-/* Note: changed here to have more mnemonic name than the original */
-
 static double
-inverse_error_func(y)
-    double y;
+inverse_error_func(double y)
 {
     double x = 0.0;    /* The output */
     double z = 0.0;    /* Intermadiate variable */
@@ -1992,15 +1866,14 @@ inverse_error_func(y)
 
 /* Implementation of Lamberts W-function which is defined as
  * w(x)*e^(w(x))=x
- * Implementation by Gunter Kuhnle, gk@uni-leipzig.de 
+ * Implementation by Gunter Kuhnle, gk@uni-leipzig.de
  * Algorithm originally developed by
  * KEITH BRIGGS, DEPARTMENT OF PLANT SCIENCES,
  * e-mail:kmb28@cam.ac.uk
  * http://epidem13.plantsci.cam.ac.uk/~kbriggs/W-ology.html */
 
-static double 
-lambertw(x)
-    double x;
+static double
+lambertw(double x)
 {
     double p, e, t, w, eps;
     int i;
@@ -2035,8 +1908,7 @@ lambertw(x)
 }
 
 void
-f_lambertw(arg)
-union argument *arg;
+f_lambertw(union argument *arg)
 {
     struct value a;
     double x;
